module Tax_Stuffs

#!/usr/bin/env ruby
require 'sketchup.rb'
require "extensions.rb"
require "bigdecimal/math"
include BigMath

(#Create Tax submenu
@TaxSubmenu = UI.menu("Plugins").add_submenu("Tax Make Parts")

#MARINER CHAIN
@TaxSubmenu.add_item("Mariner Chain") {
#Define camera view
camera = Sketchup::Camera.new([0, 0, 10], [0, 0, 0], Y_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.mariner_dialog
}

#Roller CHAIN
@TaxSubmenu.add_item("Roller Chain") {
#Define camera view
camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.roller_dialog
}

#SPU GEAR
@TaxSubmenu.add_item("Spur Gear") {
#Define camera view
camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.spur_dialog
}

#Linear Spring
@TaxSubmenu.add_item("Linear Spring") {
#Define camera view
camera = Sketchup::Camera.new([2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.linear_spiral_dialog
}

#Spheric Spring
@TaxSubmenu.add_item("Spheric Spring") {
#Define camera view
camera = Sketchup::Camera.new([2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.spheric_spiral_dialog
}

#Adjustable Suspension
@TaxSubmenu.add_item("Adjustable Suspension") {
#Define camera view
camera = Sketchup::Camera.new([15, -15, 15], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.adjustable_suspension_dialog
}

#Bicycle Wheel
@TaxSubmenu.add_item("Bicycle Wheel") {
#Define camera view
camera = Sketchup::Camera.new([0, -200, 0], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

UI.messagebox("Don't forget to save your project.")

Tax_Stuffs.bicycle_wheel_dialog
}

#RailWay Along
@TaxSubmenu.add_item("Railway Along") {
UI.messagebox("Don't forget to save your project.")
Tax_Stuffs.railway_along_dialog
}

)


#General instance variables
@PI = Math::PI
@PI50 = PI(50)


def self.mariner_dialog
#handle model and entities collection
model = Sketchup.active_model
entities = model.active_entities

prompts = ["Nominal Diameter:", "Inside Length:", "Inside Width:", "Chain type:", "Detail (sides):", "Copies in row:"]
defaults = [5.00, 35.00, 10.00, "Stud", 32, 5]
list = ["", "", "", "Open|Stud", "", ""]
input = UI.inputbox(prompts, defaults, list, "Make Mariner Chain")

#Get input values
if input
user_input1, user_input2, user_input3, user_input4, user_input5, user_input6 = input;
else
 abort("Make Mariner Chain error: MxV.000 - Operation canceled.")
end

#Print input log
print 'Make Mariner Chain operation:'
print 'Input log: ' + input[0].to_s + ' | ' + input[1].to_s + ' | ' + input[2].to_s + ' | ' + input[3].to_s + ' | ' + input[4].to_s + ' | ' + input[5].to_s

print 'Make Mariner Chain - Debug point 1/5 ok'


#Define camera view
camera = Sketchup::Camera.new([0, 0, 10], [0, 0, 0], Y_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)


  #Get variables input
  @ND = user_input1.to_f
  @ILength = user_input2.to_f
		@IWidth = user_input3.to_f
  @Detail = user_input5.to_i
  @Type = user_input4.to_s
  @Copies = user_input6.to_f
  #Conversion
  @Scale = 1000.000
  @ND = (@ND * @Scale).mm
  @ILength = (@ILength * @Scale).mm
		@IWidth = (@IWidth * @Scale).mm

(#VERIFICATIONS

#Length < width
if (@ILength < @IWidth)
UI.messagebox("Length can't be smaller than width.")
abort("Make Parts error: Length can't be smaller than width.")
end

#ND mustbe smaller than width
if (@ND > @IWidth)
UI.messagebox("Diameter can't be bigger than width.")
abort("Make Parts error: Diameter can't be bigger than width.")
end

#length must be at least 2*ND
if (@ILength < (@ND*2))
UI.messagebox('Length must be at least 2 * diameter.')
abort('Make Parts error: Length must be at least 2 * diameter.')
end

#if STUD, length must be at least 3*ND
if (@Type == "Stud") and (@ILength < (@ND*3))
 UI.messagebox('When using stud, length must be at least 3 * diameter.')
 abort('Make Parts error: When using stud, length must be at least 3 * diameter.')
end

)

print 'Make Mariner Chain - Debug point 2/5 ok'

(#Math --- 

#PT1
@pt1x = ((@ILength - @IWidth) / 2) * (-1)
@pt1y = (@IWidth + @ND) / 2

#PT2
@pt2x = (@ILength - @IWidth) / 2 
@pt2y = (@IWidth + @ND) / 2

#PT3
@pt3x = ((@ILength - @IWidth) / 2) * (-1)
@pt3y = ((@IWidth + @ND) / 2) * (-1)

#PT4
@pt4x = (@ILength - @IWidth) / 2 
@pt4y = ((@IWidth + @ND) / 2) * (-1)

@raio = (@IWidth + @ND) / 2

@line1 = []
@line1[0] = [@pt1x, @pt1y, 0]
@line1[1] = [@pt2x, @pt2y, 0]

@line2 = []
@line2[0] = [@pt3x, @pt3y, 0]
@line2[1] = [@pt4x, @pt4y, 0]
)

model = Sketchup.active_model
entities = model.active_entities

   #MODELING

   #Group 1 - Construction 
   @ConstPath = Sketchup.active_model.entities.add_group
   @L1 = @ConstPath.entities.add_line @line1
   @L2 = @ConstPath.entities.add_line @line2
   @A1 = @ConstPath.entities.add_arc [-@pt1x, 0, 0], [0,@pt3y,0], [0,0,1], @raio, 0.degrees, 180.degrees, @Detail
   @A2 = @ConstPath.entities.add_arc [@pt1x, 0, 0], [0,@pt1y,0], [0,0,1], @raio, 0.degrees, 180.degrees, @Detail
   
   #Group 2 - BASIC ELO
   @EloGroup = Sketchup.active_model.entities.add_group
   @C1 = @EloGroup.entities.add_circle([0,@pt1y,0], [1, 0, 0] , @ND/2, @Detail)
   @F1 = @EloGroup.entities.add_face(@C1)
   status = @F1.material = [153, 153, 255]
   status = @F1.back_material = [153, 153, 255]
   @Path = @L1.all_connected
   @ELO = @F1.followme(@Path)
   status = @ConstPath.erase!
   @EloFinal = @EloGroup

print 'Make Mariner Chain - Debug point 3/5 ok'

#Making STUD link
if (@Type == "Stud")
   #Group 3 - STUD
   @StudGroup = Sketchup.active_model.entities.add_group
   @C2 = @StudGroup.entities.add_circle([0,@pt3y,0], [0, 1, 0] , @ND/2, @Detail)
   @F2 = @StudGroup.entities.add_face(@C2)
   status = @F2.material = [153, 153, 255]
   status = @F2.back_material = [153, 153, 255]
   status = @F2.pushpull((@IWidth + @ND), true)

   model = Sketchup.active_model
   entities = model.active_entities
   @Temp = @StudGroup.union(@EloGroup)

        #Changing axis
        @EloFinal = Sketchup.active_model.entities.add_group
        @L1 = @EloFinal.entities.add_line @line1
        @EloFinal.entities.add_instance(@Temp.entities.parent, @Temp.transformation*@EloFinal.transformation)
        status = @L1.erase!
        status = @Temp.erase!
        @EloFinal.entities[0].explode
end


#Putting 2nd chain link
@EloFinal = @EloFinal.to_component

@EloFinal2 = @EloFinal.copy
@EloFinal2 = @EloFinal2.move! [(@ILength), 0, 0]


   model = Sketchup.active_model
   entities = model.active_entities
   @MarinerDuo = Sketchup.active_model.entities.add_group
   @MarinerDuo.entities.add_instance(@EloFinal.definition, @EloFinal.transformation*@MarinerDuo.transformation)
   @MarinerDuo.entities.add_instance(@EloFinal2.definition, @EloFinal2.transformation*@MarinerDuo.transformation)
   @EloFinal.erase!
   @EloFinal2.erase!
   entities.transform_entities(Geom::Transformation.rotation([0,0,0],[1,0,0],90.degrees), @MarinerDuo.entities[1])

   @Elo1 = @MarinerDuo.entities[0]
   @Elo2 = @MarinerDuo.entities[1]

print 'Make Mariner Chain - Debug point 4/5 ok'

if @Copies > 0
 model = Sketchup.active_model
 entities = model.active_entities
 @WholeChain = Sketchup.active_model.entities.add_group
 @WholeChain.entities.add_instance(@MarinerDuo.definition, @MarinerDuo.transformation*@WholeChain.transformation)
 @MarinerDuo.erase!

  for step in 1..@Copies
  @Extend = @WholeChain.entities[0].copy
  @Extend = @Extend.move! [(@ILength)*2*step, 0, 0]
  end
 
 @constpoint = @WholeChain.entities.add_cpoint [0, 0, 0]
 #Reset scale
 scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
 @WholeChain.transformation *= scale_transform
 
else
 @constpoint = @MarinerDuo.entities.add_cpoint [0, 0, 0]
 #Reset scale
 scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
 @MarinerDuo.transformation *= scale_transform
end

print 'Make Mariner Chain - Debug point 5/5 ok'

#End Message
@EndMessage = "Success. :D \n Object placed at origin[0,0,0]."
UI.messagebox(@EndMessage)

print "Success. \n \n"
end

def self.roller_dialog
#handle model and entities collection
model = Sketchup.active_model
entities = model.active_entities

@Pitch = 8.00
@RollerDiameter = 4.70
@RollerWidth = 4.80
@RollerTickness = 0.20
@PinDiameter = 3.25
@PlateTickness = 1.00
@Detail = 32
@RollerOutRatio = 1.60
@R1R2Ratio = 1.00
@Smooth = "Yes"
@bevelfactor = 0.20
@Links = 10
@MakeRef2 = "Yes"


@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["Pitch:", "Roller Diameter:", "Roller Width:", "Roller Tickness:", "Pin Diameter:", "Plate Tickness:", "Detail (sides):", "Roller-Out Ratio:", "Rin-Rout Ratio:", "Smooth?", "Bevel Factor:", "Copies in row:", "Construct ref.?:"]
defaults = [@Pitch, @RollerDiameter, @RollerWidth, @RollerTickness, @PinDiameter, @PlateTickness, @Detail, @RollerOutRatio, @R1R2Ratio, @Smooth, @bevelfactor, @Links, @MakeRef2]
list = ["", "", "", "", "", "", "", "", "", "Yes|No", "", "", "Yes|No"]
input = UI.inputbox(prompts, defaults, list, "Make Roller Chain")

#Get input values
if input
@Pitch, @RollerDiameter, @RollerWidth, @RollerTickness, @PinDiameter, @PlateTickness, @Detail, @RollerOutRatio, @R1R2Ratio, @Smooth, @bevelfactor, @Links, @MakeRef2 = input;
else
 abort("Make Roller Chain error: RxV.000 - Operation canceled.")
end

#Print input log
print 'Make Roller Chain operation:'
print 'Input log: ' + input[0].to_s + ' | ' + input[1].to_s + ' | ' + input[2].to_s + ' | ' + input[3].to_s + ' | ' + input[4].to_s + ' | ' + input[5].to_s + ' | ' + input[6].to_s + ' | ' + input[7].to_s + ' | ' + input[8].to_s + ' | ' + input[9].to_s + ' | ' + input[10].to_s + ' | ' + input[11].to_s + ' | ' + input[12].to_s

print 'Make Roller Chain - Debug point 1/7 ok'

#Timer start ------------------------------------------------------------------------------
@start_time = Time.now

(#Define camera view
camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#Get variables input
@Pitch = @Pitch.to_f
@RollerDiameter = @RollerDiameter.to_f
@RollerWidth = @RollerWidth.to_f
@RollerTickness = @RollerTickness.to_f
@PinDiameter = @PinDiameter.to_f
@PlateTickness = @PlateTickness.to_f
@Detail = @Detail.to_i
@RollerOutRatio = @RollerOutRatio.to_f
@R1R2Ratio = @R1R2Ratio.to_f
@Smooth = @Smooth.to_s
@bevelfactor = @bevelfactor.to_f
@Links = @Links.to_i
@MakeRef2 = @MakeRef2.to_s

#math
@BrushingWidth = (@RollerWidth + (2*@PlateTickness))
@PinLength = (@RollerWidth + (4*@PlateTickness))
@BrushingDiameter = (@RollerDiameter - (@RollerTickness*2))
@BrushingTickness = (@BrushingDiameter - @PinDiameter)/2

   if (@Smooth == "No")
   @bevelfactor = 0.25
   end
)

print 'Make Roller Chain - Debug point 2/7 ok'
(#VERIFICATIONS
if (@Pitch <= 0) or (@Pitch <= @RollerDiameter) or ((@RollerDiameter * @RollerOutRatio) > @Pitch)
UI.messagebox("error: RxV.001 - Pitch is to small or roller diameter is too large.")
warn("Make Parts error: RxV.001")
@Verification = "NOT OK"
end

if (@PlateTickness <= 0) or (((@RollerDiameter * @RollerOutRatio)-@RollerDiameter)/2 <= 0)
UI.messagebox("error: RxV.002 - Plate tickness or roller out ratio is too low.")
warn("Make Parts error: RxV.002")
@Verification = "NOT OK"
end

if (@Smooth == "Yes") and ( (((@RollerDiameter * @RollerOutRatio)-@RollerDiameter)/4) <= (@PlateTickness * @bevelfactor))
UI.messagebox("error: RxV.003 - Plate tickness or bevel factor is too large or roller out ratio is too low.")
warn("Make Parts error: RxV.003")
@Verification = "NOT OK"
end

if (@RollerWidth <= 0)
UI.messagebox("error: RxV.004 - Roller width is too small.")
warn("Make Parts error: RxV.004")
@Verification = "NOT OK"
end

if (@PinDiameter <= 0)
UI.messagebox("error: RxV.005 - Pin diameter is too small.")
warn("Make Parts error: RxV.005")
@Verification = "NOT OK"
end

if (@BrushingDiameter <= @PinDiameter)
UI.messagebox("error: RxV.006 - No space for brushes; reduce pin diameter and/or roller thickness or increase roller diameter")
warn("Make Parts error: RxV.006")
@Verification = "NOT OK"
end

if (@RollerDiameter <= 0) or (@RollerDiameter <= @BrushingDiameter) or (@RollerDiameter >= @Pitch)
UI.messagebox("error: RxV.007 - Roller diameter too short or too big.")
warn("Make Parts error: RxV.007")
@Verification = "NOT OK"
end

if (@Detail <= 3)
UI.messagebox("error: RxV.008 - Min. detail level = 3.")
warn("Make Parts error: RxV.008")
@Verification = "NOT OK"
end

if (@R1R2Ratio <= 0)
UI.messagebox("error: RxV.009 - Rin-Rout ratio is too low.")
warn("Make Parts error: RxV.009")
@Verification = "NOT OK"
end

if (@Smooth == "Yes") and ((@bevelfactor <= 0) or (@bevelfactor > 0.5))
UI.messagebox("error: RxV.010 - Bevel factor out of (0 < x <= 0.5) range.")
warn("Make Parts error: RxV.010")
@Verification = "NOT OK"
end


@OuterDiameter = @RollerDiameter * @RollerOutRatio
@R1 = @OuterDiameter / 2
@R2 = @R1 * @R1R2Ratio

if ( (@R1+@R2) < (@Pitch/2) )
UI.messagebox("error: RxV.011 - Rin-Rout ratio is to low.")
warn("Make Parts error: RxV.011")
@Verification = "NOT OK"
end


@IntersectAngle = Math.acos(@Pitch / (2*(@R1+@R2))).radians
@R2y = Math.sin(@IntersectAngle.degrees) * (@R1+@R2)

if ( ((@R2y - @R2) - (@PlateTickness * @bevelfactor)) < @PinDiameter/4 )
UI.messagebox("error: RxV.012 - Pitch is too high or Rin-Rout ratio is to low")
warn("Make Parts error: RxV.012")
@Verification = "NOT OK"
end

)
end

print 'Make Roller Chain - Debug point 3/7 ok'
UI.messagebox("It can take a few minutes, please wait.")

(#Conversion
@Pitch = @Pitch.mm
@RollerDiameter = @RollerDiameter.mm
@RollerWidth = @RollerWidth.mm
@RollerTickness = @RollerTickness.mm
@PinDiameter = @PinDiameter.mm
@PlateTickness = @PlateTickness.mm
@BrushingWidth = @BrushingWidth.mm
@PinLength = @PinLength.mm
@BrushingDiameter = @BrushingDiameter.mm
@BrushingTickness = @BrushingTickness.mm
)


#MODELING

 def self.plates(pitch, brushingdiameter, rollerdiameter, platetickness, rolleroutratio, r1r2ratio, detail, bevelfactor, smooth)

@HoleRadius = (brushingdiameter/2)
@Bevel = platetickness * bevelfactor
@OuterDiameter = rollerdiameter * rolleroutratio
@R1 = @OuterDiameter / 2
@R2 = @R1 * r1r2ratio
@IntersectAngle = Math.acos(pitch / (2*(@R1+@R2))).radians
@R2y = Math.sin(@IntersectAngle.degrees) * (@R1+@R2)

#Make group
model = Sketchup.active_model
entities = model.active_entities
@Plate = Sketchup.active_model.entities.add_group

#outers of 1st side
@C1 = @Plate.entities.add_circle([pitch/2, 0, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@C2 = @Plate.entities.add_circle([-pitch/2, 0, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@OuterArc1 = @Plate.entities.add_arc [pitch/2, 0, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2 = @Plate.entities.add_arc [-pitch/2, 0, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArc = @Plate.entities.add_arc [0, 0, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArc = @Plate.entities.add_arc [0, 0, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPath = @SuperiorArc[0].all_connected
@PlateFace = @Plate.entities.add_face(@OuterPath)

@FTR1 = @Plate.entities.add_face(@C1)
status = @FTR1.erase!
@FTR2 = @Plate.entities.add_face(@C2)
status = @FTR2.erase!

if @PlateFace.normal.y == -1
@PlateFace.reverse!
end

status = @PlateFace.pushpull(platetickness, true)



if (@Smooth == "Yes")
model = Sketchup.active_model
entities = model.active_entities

@HoleRadius = (brushingdiameter/2) + @Bevel
@R1 = @R1 - @Bevel
@R2 = @R2 + @Bevel

#inners of 1st side
@C1s = @Plate.entities.add_circle([pitch/2, 0, 0], [0, 1, 0] , @HoleRadius, detail)
@C2s = @Plate.entities.add_circle([-pitch/2, 0, 0], [0, 1, 0] , @HoleRadius, detail)
@OuterArc1s = @Plate.entities.add_arc [pitch/2, 0, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2s = @Plate.entities.add_arc [-pitch/2, 0, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArcs = @Plate.entities.add_arc [0, 0, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArcs = @Plate.entities.add_arc [0, 0, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPaths = @SuperiorArcs[0].all_connected
@PlateFaces = @Plate.entities.add_face(@OuterPaths)

#Make bevel 1st side
transform = Geom::Transformation.new([0,@Bevel,0])
@Plate.entities.transform_entities(transform, @OuterPath)
@Plate.entities.transform_entities(transform, @C1)
@Plate.entities.transform_entities(transform, @C2)


#inners of 2nd side
@C1ss = @Plate.entities.add_circle([pitch/2, platetickness, 0], [0, 1, 0] , @HoleRadius, detail)
@C2ss = @Plate.entities.add_circle([-pitch/2, platetickness, 0], [0, 1, 0] , @HoleRadius, detail)
@OuterArc1ss = @Plate.entities.add_arc [pitch/2, platetickness, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc2ss = @Plate.entities.add_arc [-pitch/2, platetickness, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArcss = @Plate.entities.add_arc [0, platetickness, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArcss = @Plate.entities.add_arc [0, platetickness, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2

@OuterPathss = @SuperiorArcss[0].all_connected
@PlateFacess = @Plate.entities.add_face(@OuterPathss)


#Outers of 2nd side
@R1 = @OuterDiameter / 2
@R2 = @R1 * r1r2ratio

@C12 = @Plate.entities.add_circle([pitch/2, platetickness, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@C22 = @Plate.entities.add_circle([-pitch/2, platetickness, 0], [0, 1, 0] , (brushingdiameter/2), detail)
@OuterArc12 = @Plate.entities.add_arc [pitch/2, platetickness, 0], [-1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@OuterArc22 = @Plate.entities.add_arc [-pitch/2, platetickness, 0], [1,0,0], [0,1,0], @R1, @IntersectAngle.degrees, (360-@IntersectAngle).degrees, detail
@SuperiorArc2 = @Plate.entities.add_arc [0, platetickness, @R2y], [1,0,0], [0,1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2
@InferiorArc2 = @Plate.entities.add_arc [0, platetickness, -@R2y], [1,0,0], [0,-1,0], @R2, @IntersectAngle.degrees, (180-@IntersectAngle).degrees, detail/2


#Make bevel 2nd side
Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@SuperiorArc2)
Sketchup.active_model.selection.add(@InferiorArc2)
Sketchup.active_model.selection.add(@OuterArc12)
Sketchup.active_model.selection.add(@OuterArc22)

transform = Geom::Transformation.new([0,-@Bevel,0])
@Plate.entities.transform_entities(transform, Sketchup.active_model.selection)
@Plate.entities.transform_entities(transform, @C12)
@Plate.entities.transform_entities(transform, @C22)

Sketchup.active_model.selection.clear
end

@Plate = @Plate.move! [0, -(platetickness/2), 0]

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Plate.entities.parent, @Plate.transformation*@Temp.transformation)
status = @L1.erase!
status = @Plate.erase!
@Temp.entities[0].explode
@Plate = @Temp
end

 def self.tube(diameter, tickness, width, detail, bevelfactor, smooth)
 
   #get variables
   @Bevel = tickness * bevelfactor
   @Radius = diameter/2
   
   #Make group
   model = Sketchup.active_model
   entities = model.active_entities
   @Tube = Sketchup.active_model.entities.add_group

   @Tc1 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , @Radius, detail)
   @Tc2 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-tickness), detail)
   @Tf2 = @Tube.entities.add_face(@Tc2)
   @Tf1 = @Tube.entities.add_face(@Tc1)
   status = @Tf2.erase!
   status = @Tf1.pushpull(width, true)
    
   if (@Smooth == "Yes")
   #internal bevel circles face 1
    @C1 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-@Bevel), detail)
    @C2 = @Tube.entities.add_circle([0,-width/2,0], [0, 1, 0] , (@Radius-tickness+@Bevel), detail)
   #internal bevel circles face 2
    @C3 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-@Bevel), detail)
    @C4 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-tickness+@Bevel), detail)

    #move external bevels face 1
    transform = Geom::Transformation.new([0,@Bevel,0])
    @Tube.entities.transform_entities(transform, @Tc1)
    @Tube.entities.transform_entities(transform, @Tc2)

    #move external bevels face 2
    @Tc12 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , @Radius, detail)
    @Tc22 = @Tube.entities.add_circle([0,width/2,0], [0, 1, 0] , (@Radius-tickness), detail)
    Sketchup.active_model.selection.clear
    Sketchup.active_model.selection.add(@Tc12)
    Sketchup.active_model.selection.add(@Tc22)
    transform = Geom::Transformation.new([0,-@Bevel,0])
    @Tube.entities.transform_entities(transform, Sketchup.active_model.selection)
    Sketchup.active_model.selection.clear
   end
   
#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Tube.entities.parent, @Tube.transformation*@Temp.transformation)
status = @L1.erase!
status = @Tube.erase!
@Temp.entities[0].explode
@Tube = @Temp
 end

 def self.pin(diameter, length, detail, bevelfactor, smooth)
 @ExtentionFactor = 0.5
 @Extension = @ExtentionFactor * diameter
 @Scale = 100.00

  @Bevel1 = 0.0
  @Bevel2 = bevelfactor * (diameter/2)
  if (@Smooth == "Yes")
  @Bevel1 = bevelfactor * (diameter/2)
  end
 

 #Make group
 model = Sketchup.active_model
 entities = model.active_entities
 @Pin = Sketchup.active_model.entities.add_group


 @ProfileContourn = [
 [0, -((length/2)+@Extension)* @Scale, 0],
 [0, (length/2)* @Scale, 0],
 [((diameter/2)-@Bevel1)* @Scale, (length/2)* @Scale, 0],
 [(diameter/2)* @Scale, ((length/2)-@Bevel2)* @Scale, 0],

 [(diameter/2)* @Scale, -(length/2)* @Scale, 0],
 [(diameter/3)* @Scale, -(length/2)* @Scale, 0],
 [(diameter/3)* @Scale, -((length/2)+(diameter/6))* @Scale, 0],
 [(diameter/2)* @Scale, -((length/2)+(diameter/6))* @Scale, 0],

 [(diameter/2)* @Scale, -((length/2)+@Extension-@Bevel2)* @Scale, 0],
 [((diameter/2)-@Bevel1)* @Scale, -((length/2)+@Extension)* @Scale, 0],
 [0, -((length/2)+@Extension)* @Scale, 0]
 ]

 @Pf = @Pin.entities.add_face(@ProfileContourn)
 
 @Pc = @Pin.entities.add_circle([0,0,0], [0, 1, 0] , (diameter* @Scale), detail)
 @Pfollow = @Pf.followme(@Pc)

   @Pin_Counter = 0
   while @Pin_Counter < @Pc.length
   status = @Pc[@Pin_Counter].erase!
   @Pin_Counter += 1
   end
 
#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Pin.entities.parent, @Pin.transformation*@Temp.transformation)
status = @L1.erase!
status = @Pin.erase!
@Temp.entities[0].explode
@Pin = @Temp

#Make hole
@Hole = Sketchup.active_model.entities.add_group
@C1 = @Hole.entities.add_circle([0,-((length/2)+(diameter/6)) * @Scale, -diameter * @Scale], [0, 0, 1] , (diameter/6) * @Scale, detail)
@F1 = @Hole.entities.add_face(@C1)
status = @F1.pushpull((diameter*2) * @Scale, true)
@Pin = @Hole.subtract(@Pin)

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Pin)
@Pin.transformation = Geom::Transformation.scaling([0,0,0], (1.0/@Scale).to_f) * @Pin.transformation

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Pin.entities.parent, @Pin.transformation*@Temp.transformation)
status = @L1.erase!
status = @Pin.erase!
@Temp.entities[0].explode
@Pin = @Temp
 end

 def self.lock(inrad, outrad, tickness, detail, bevelfactor, smooth)
 #Make group
 model = Sketchup.active_model
 entities = model.active_entities
 @Lock = Sketchup.active_model.entities.add_group
 @Scale = 100.00

@R1 = (inrad)
@R2 = (outrad)
@AngA = 100.degrees
@AngB = 360.degrees
@AngC = 10.degrees
@Thick = tickness
@Detail = detail
@Bevel = @Thick * bevelfactor

@A1 = @Lock.entities.add_arc [0, 0, 0], [0,0,-1], [0,1,0], @R2* @Scale, @AngA, @AngB, detail
@A2 = @Lock.entities.add_arc [0, 0, 0], [0,0,-1], [0,1,0], @R1* @Scale, @AngA, @AngB, detail
@L1 = @Lock.entities.add_line [0,0,-@R1* @Scale], [0,0,-@R2* @Scale]
@L2 = @Lock.entities.add_line [(-Math.cos(@AngC)*@R1)* @Scale,0, (Math.sin(@AngC)*@R1)* @Scale], [-(Math.cos(@AngC)*@R2)* @Scale,0, (Math.sin(@AngC)*@R2)* @Scale]

@Path = @L1.all_connected
@F1 = @Lock.entities.add_face(@Path)


if @F1.normal.y == -1
@F1.reverse!
end

status = @F1.pushpull(-@Thick* @Scale, true)

  if (@Smooth == "Yes")
  @A1o = @Lock.entities.add_arc [0, -tickness* @Scale, 0], [0,0,-1], [0,1,0], @R2* @Scale, @AngA, @AngB, detail
  @A1b = @Lock.entities.add_arc [0, -tickness* @Scale, 0], [0,0,-1], [0,1,0], (@R2-@Bevel)* @Scale, @AngA, @AngB, detail
  Sketchup.active_model.selection.clear
  Sketchup.active_model.selection.add(@A1o)
  transform = Geom::Transformation.new([0,@Bevel* @Scale,0])
  @Lock.entities.transform_entities(transform, @A1o)
  Sketchup.active_model.selection.clear
  end

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Lock)
@Lock.transformation = Geom::Transformation.scaling([0,0,0], (1.0/@Scale).to_f) * @Lock.transformation

#Changing axis
@Temp = Sketchup.active_model.entities.add_group
@L1 = @Temp.entities.add_line [0,0,0], [1,1,1]
@Temp.entities.add_instance(@Lock.entities.parent, @Lock.transformation*@Temp.transformation)
status = @L1.erase!
status = @Lock.erase!
@Temp.entities[0].explode
@Lock = @Temp
 end


(#ASSEMBLE
model = Sketchup.active_model
entities = model.active_entities


print 'Make Roller Chain - Debug point 4/7 ok'
#Make BnR elements - - - - - - - - - - - - - - - - - - -
@PlateX1 = Tax_Stuffs.plates(@Pitch, @BrushingDiameter, @RollerDiameter, @PlateTickness, @RollerOutRatio, @R1R2Ratio, @Detail, @bevelfactor, @Smooth)
@PlateX1 = @PlateX1.to_component
@PlateX1 = @PlateX1.move! [0, (-(@RollerWidth+@PlateTickness)/2), 0]
@PlateX2 = @PlateX1.copy
@PlateX2 = @PlateX2.move! [0, (@RollerWidth+@PlateTickness)/2, 0]

@TubeX1 = Tax_Stuffs.tube(@BrushingDiameter, @BrushingTickness, @BrushingWidth, @Detail, @bevelfactor, @Smooth)
@TubeX1 = @TubeX1.to_component
@TubeX1 = @TubeX1.move! [(-@Pitch/2), 0, 0]
@TubeX2 = @TubeX1.copy
@TubeX2 = @TubeX2.move! [(@Pitch/2), 0, 0]

@TubeX3 = Tax_Stuffs.tube(@RollerDiameter, @RollerTickness, @RollerWidth, @Detail, @bevelfactor, @Smooth)
@TubeX3 = @TubeX3.to_component
@TubeX3 = @TubeX3.move! [(-@Pitch/2), 0, 0]
@TubeX4 = @TubeX3.copy
@TubeX4 = @TubeX4.move! [(@Pitch/2), 0, 0]


#Make group 1 Brushes and Rollers
@BnR = Sketchup.active_model.entities.add_group
@L1 = @BnR.entities.add_line [0,0,0], [1,1,1]
@BnR.entities.add_instance(@PlateX1.definition, @PlateX1.transformation*@BnR.transformation)
@BnR.entities.add_instance(@PlateX2.definition, @PlateX2.transformation*@BnR.transformation)
@BnR.entities.add_instance(@TubeX1.definition, @TubeX1.transformation*@BnR.transformation)
@BnR.entities.add_instance(@TubeX2.definition, @TubeX2.transformation*@BnR.transformation)
@BnR.entities.add_instance(@TubeX3.definition, @TubeX3.transformation*@BnR.transformation)
@BnR.entities.add_instance(@TubeX4.definition, @TubeX4.transformation*@BnR.transformation)
status = @L1.erase!
status = @PlateX1.erase!
status = @PlateX2.erase!
status = @TubeX1.erase!
status = @TubeX2.erase!
status = @TubeX3.erase!
status = @TubeX4.erase!

#Put colors
status = @BnR.entities[0].material = [100, 40, 255]
status = @BnR.entities[1].material = [100, 40, 255]
status = @BnR.entities[2].material = [70, 100, 150]
status = @BnR.entities[3].material = [70, 100, 150]
status = @BnR.entities[4].material = [170, 140, 90]
status = @BnR.entities[5].material = [170, 140, 90]

@BnR = @BnR.to_component


print 'Make Roller Chain - Debug point 5/7 ok'
#Make PnL elements - - - - - - - - - - - - - - - - - - -
@PlateX1 = Tax_Stuffs.plates(@Pitch, @PinDiameter, @RollerDiameter, @PlateTickness, @RollerOutRatio, @R1R2Ratio, @Detail, @bevelfactor, @Smooth)
@PlateX1 = @PlateX1.to_component
@PlateX1 = @PlateX1.move! [@Pitch, -((@RollerWidth/2)+(@PlateTickness*1.5)), 0]
@PlateX2 = @PlateX1.copy
@PlateX2 = @PlateX2.move! [@Pitch, ((@RollerWidth/2)+(@PlateTickness*1.5)), 0]

@PinX1 = Tax_Stuffs.pin(@PinDiameter, @PinLength, @Detail, @bevelfactor, @Smooth)
@PinX1 = @PinX1.to_component
@PinX1 = @PinX1.move! [@Pitch/2, 0, 0]
@PinX2 = @PinX1.copy
@PinX2 = @PinX2.move! [@Pitch*1.5, 0, 0]

@LockX1 = Tax_Stuffs.lock((@PinDiameter/3), @PinDiameter*0.8, (@PinDiameter/6), @Detail, @bevelfactor, @Smooth)
@LockX1 = @LockX1.to_component
@LockX1 = @LockX1.move! [@Pitch/2, -(@PinLength/2), 0]
@LockX2 = @LockX1.copy
@LockX2 = @LockX2.move! [@Pitch*1.5, -(@PinLength/2), 0]

#Make group 2 Plates, Pins and Locks
@PnL = Sketchup.active_model.entities.add_group
@L1 = @PnL.entities.add_line [0,0,0], [1,1,1]
@PnL.entities.add_instance(@PlateX1.definition, @PlateX1.transformation*@PnL.transformation)
@PnL.entities.add_instance(@PlateX2.definition, @PlateX2.transformation*@PnL.transformation)
@PnL.entities.add_instance(@PinX1.definition, @PinX1.transformation*@PnL.transformation)
@PnL.entities.add_instance(@PinX2.definition, @PinX2.transformation*@PnL.transformation)
@PnL.entities.add_instance(@LockX1.definition, @LockX1.transformation*@PnL.transformation)
@PnL.entities.add_instance(@LockX2.definition, @LockX2.transformation*@PnL.transformation)
status = @L1.erase!
status = @PlateX1.erase!
status = @PlateX2.erase!
status = @PinX1.erase!
status = @PinX2.erase!
status = @LockX1.erase!
status = @LockX2.erase!

#Put colors
status = @PnL.entities[0].material = [100, 40, 255]
status = @PnL.entities[1].material = [100, 40, 255]
status = @PnL.entities[2].material = [250, 180, 230]
status = @PnL.entities[3].material = [250, 180, 230]
status = @PnL.entities[4].material = [80, 80, 80]
status = @PnL.entities[5].material = [80, 80, 80]

@PnL = @PnL.to_component

print 'Make Roller Chain - Debug point 6/7 ok'
#Make Unit group 0
@PatternUnit = Sketchup.active_model.entities.add_group
@L1 = @PatternUnit.entities.add_line [0,0,0], [1,1,1]
@PatternUnit.entities.add_instance(@BnR.definition, @BnR.transformation*@PatternUnit.transformation)
@PatternUnit.entities.add_instance(@PnL.definition, @PnL.transformation*@PatternUnit.transformation)
status = @L1.erase!
status = @BnR.erase!
status = @PnL.erase!


#Make contruction references
if (@MakeRef2 == "Yes")
model = Sketchup.active_model
entities = model.active_entities
@Refs = @PatternUnit.entities.add_group
@CenterPoint0 = @Refs.entities.add_cpoint [-@Pitch/2, 0, 0]
@CenterPoint1 = @Refs.entities.add_cpoint [0, 0, 0]
@CenterPoint2 = @Refs.entities.add_cpoint [@Pitch/2, 0, 0]
@CenterPoint3 = @Refs.entities.add_cpoint [@Pitch, 0, 0]
@CenterPoint4 = @Refs.entities.add_cpoint [@Pitch*1.5, 0, 0]
@LineSize = (@RollerWidth + @PlateTickness*4)
@RefLine0 = @Refs.entities.add_line [-@Pitch/2,@LineSize,0], [-@Pitch/2,-@LineSize,0]
@RefLine1 = @Refs.entities.add_line [@Pitch/2,@LineSize,0], [@Pitch/2,-@LineSize,0]
@RefLine2 = @Refs.entities.add_line [@Pitch*1.5,@LineSize,0], [@Pitch*1.5,-@LineSize,0]
end

@PatternUnit = @PatternUnit.to_component
)


(# links in row
if @Links > 0
@Whole = Sketchup.active_model.entities.add_group
@Whole.entities.add_instance(@PatternUnit.definition, @PatternUnit.transformation*@Whole.transformation)
 for step in 1..@Links
 @Extend = @PatternUnit.copy
 @Extend = @Extend.move! [(@Pitch*2)*step, 0, 0]
 @Whole.entities.add_instance(@Extend.definition, @Extend.transformation*@Whole.transformation)
 status = @Extend.erase!
 end
 status = @PatternUnit.erase!
end
)

print 'Make Roller Chain - Debug point 7/7 ok'

#--------------------------------------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time

#End Message
@EndMessage = "Success. (" + @Total_Time.to_s + " seconds)"
UI.messagebox(@EndMessage)
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
end

def self.spur_dialog
#-------------------------------------------------------------------------------
prompts = ["Pitch:", "Teeths:", "Pressure angle:", "Clearance:", "Width:", "Hole diameter:", "Detail (sides):", "Construct ref.?:"]
defaults = [8.00, 36, 20.00, 2.00, 4.70, 20.00, 24, "Yes"]
list = ["", "", "", "", "", "", "", "Yes|No"]
val = UI.inputbox(prompts, defaults, list, "Make Spur gear")
if val
@Pitch, @nTeeth, @PressureAngle, @Clearance, @Width, @HoleDiameter, @Detail, @MakeRef = val;
else
 abort("Make Spur Gear error: SxV.000 - Operation canceled.")
end

#Log and verification
print 'Make Spur Gear operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s


#-------------------------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([-2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera
sleep(1)

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------

#Conversion
@Scale = 10000.000
@Pitch = (@Pitch * @Scale).mm
@nTeeth = @nTeeth.to_i
@PressureAngle = @PressureAngle.degrees
# @ProfileShift = (@ProfileShift* @Scale).mm
@Clearance = (@Clearance* @Scale).mm
@Width = (@Width* @Scale).mm
@HoleDiameter = (@HoleDiameter* @Scale).mm
@Detail = @Detail.to_i
@GearColor = [100,100,100]
@PitchAngle = (360.000 / @nTeeth).degrees
@MakeRef = @MakeRef.to_s
#-------------------------------------------------------------------------------


(#VERIFICATION

#Inputs on rigth format
if (@Pitch <= 0)
UI.messagebox("error: SxV.001 - Inputs must be integer or float numbers.")
abort("Make Spur gear error: SxV.001")
end

# Pitch > 0
if (@Pitch <= 0)
UI.messagebox("error: SxV.002 - Pitch must be higher than 0.")
abort("Make Spur gear error: SxV.002")
end

# nTeeth > 2
if (@nTeeth <= 2)
UI.messagebox("error: SxV.003 - nTeeth must be higher than 2.")
abort("Make Spur gear error: SxV.003")
end

#0 < Pressure angle > 32
if (@PressureAngle <= 0) or (@PressureAngle >= 32)
UI.messagebox("error: SxV.004 - Pressure angle must be inside range (0 < pa < 32).")
abort("Make Spur gear error: SxV.004")
end

@ProfileShift = 0
# ProfileShift >= 0
if (@ProfileShift < 0)
UI.messagebox("error: SxV.005 - Profile shift must be positive.")
abort("Make Spur gear error: SxV.005")
end

#Clearance < Pitch
if (@Clearance <= 0) or (@Clearance >= @Pitch)
UI.messagebox("error: SxV.006 - Clearance must be in range (0 < C < Pitch).")
abort("Make Spur gear error: SxV.006")
end

#Width > 0
if (@Width <= 0)
UI.messagebox("error: SxV.007 - Width must be higher than 0.")
abort("Make Spur gear error: SxV.007")
end


#Pitch
@PitchDiameter = (@Pitch * @nTeeth) / Math::PI
@PitchRadius = @PitchDiameter/2

#BASE
@BaseDiameter = @PitchDiameter * Math.cos( @PressureAngle )
@BaseRadius = @BaseDiameter / 2

#Hole radius < base-cleareance radius
if (@HoleDiameter/2 >= @BaseRadius-@Clearance)
UI.messagebox("error: SxV.008 - HoleDiameter is too large.")
abort("Make Spur gear error: SxV.008")
end

#Detail > 2
if (@Detail < 3)
UI.messagebox("error: SxV.009 - Detail must be integer and higher than 2.")
abort("Make Spur gear error: SxV.009")
end
)

#-------------------------------------------------------------------------------

model = Sketchup.active_model
entities = model.active_entities

#Calculating gear basic parameters

#Pitch
@PitchDiameter = (@Pitch * @nTeeth) / Math::PI
@PitchRadius = @PitchDiameter/2

#BASE
@BaseDiameter = @PitchDiameter * Math.cos( @PressureAngle )
@BaseRadius = @BaseDiameter / 2

@Module = @PitchDiameter / @nTeeth

#TIP
@TipDiameter = @PitchDiameter + 2 * @Module * (1 + @ProfileShift)
@TipRadius = @TipDiameter / 2

@TipPressureAngle = Math.acos(@BaseDiameter / @TipDiameter).degrees
@InvAlpha = Math.tan(@PressureAngle) - @PressureAngle
@InvAlphaA = Math.tan(@TipPressureAngle.radians) - @TipPressureAngle.radians

@TopThickness = Math::PI / (2 * @nTeeth) + (2 * @ProfileShift * Math.tan(@PressureAngle)) / @nTeeth + @InvAlpha - @InvAlphaA
@TopThicknessDegrees = @TopThickness.degrees


@CrestWidth = @TipDiameter * @TopThickness
@tooththickness = 2 * ((90 / @nTeeth) + ( 360 * @ProfileShift * Math.tan(@PressureAngle) ) / Math::PI / @nTeeth)

#Angular tooth width along the base diameter.
@umax = Math.sqrt( @TipRadius * @TipRadius / @BaseRadius / @BaseRadius - 1) * 1.000 # Math.sqrt( ((R**2) / (r**2)) - 1 )
	@x1 = @BaseRadius
	@y1 = 0
	@x2 = @BaseRadius * ( Math.cos( @umax ) + @umax * Math.sin( @umax ) )
	@y2 = @BaseRadius * ( Math.sin( @umax ) - @umax * Math.cos( @umax ) )
#Distance between beginning and end of the involute curve
@d = Math.sqrt( ( @x1 - @x2 ) * ( @x1 - @x2 ) + ( @y1 - @y2 ) * ( @y1 - @y2 ) )
@cosx = (@BaseRadius * @BaseRadius + @TipRadius * @TipRadius - @d * @d) / 2 / @BaseRadius / @TipRadius
@basetooththickness = (2 * @TopThickness + 2 * Math.acos(@cosx))

print 'Make Spur Gear - Debug point 1/5 ok'

#-------------------------------------------------------------------------------

#Involute making
model = Sketchup.active_model
entities = model.active_entities
@Involute = Sketchup.active_model.entities.add_group
@Points = []


#Some math to find basic parameters of involute

#Tip arc
@AngleOffset = Math.asin(@y2/@TipRadius) #replace: ((@basetooththickness-@TopThickness)/2)
@TipArc = @Involute.entities.add_arc [0,0,0], [1, 0, 0] , [0,0,1], @TipRadius, -(@TopThickness/2), @TopThickness/2, @Detail


#involute A
for step in 0.. (@Detail)
 @angle = ((@umax / @Detail) * step) * 1.000
 @Points[step] = [@BaseRadius * (Math.cos(@angle) + @angle * Math.sin(@angle)), @BaseRadius * (Math.sin(@angle) - @angle * Math.cos(@angle)), 0]
end
@TempInv = @Involute.entities.add_group
@Inva = @TempInv.entities.add_line @Points
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], -(@AngleOffset + @TopThickness/2) )
@TempInv.transform!(transformation)
@TempInv.explode

#involute B
for a in 0.. (@Detail)
 @angle = ((@umax / @Detail) * a) * 1.000
 @Points[a] = [@BaseRadius * ( Math.cos(@angle) + ((@angle) * Math.sin(@angle)) ), -@BaseRadius * ( Math.sin(@angle) - ((@angle) * Math.cos(@angle)) ), 0]
end
@TempInv = @Involute.entities.add_group
@Invb = @TempInv.entities.add_line @Points
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], @AngleOffset + (@TopThickness/2) )
@TempInv.transform!(transformation)
@TempInv.explode

print 'Make Spur Gear - Debug point 2/5 ok'

#-------------------------------------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities


#Some math to find basic parameters of clearance arc
@ClearanceAngle = @PitchAngle - (2*@AngleOffset + @TopThickness)

@Pointa = [Math.cos( @AngleOffset + (@TopThickness/2) )*@BaseRadius, Math.sin( @AngleOffset + (@TopThickness/2) )*@BaseRadius, 0]
@Pointb = [Math.cos( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, Math.sin( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, 0]

@Corda = BigMath.sqrt( BigDecimal(((@Pointb[0] - @Pointa[0])**2) + ((@Pointb[1] - @Pointa[1])**2), 16), 20 )
@Semi_Corda = @Corda/2

@CordaCenter = [(@Pointb[0] - @Pointa[0])/2 + @Pointa[0], (@Pointb[1] - @Pointa[1])/2 + @Pointa[1], 0]
@ClearancePoint = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance), Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance), 0]

@Flecha = BigMath.sqrt(BigDecimal(( (@CordaCenter[0] - @ClearancePoint[0])**2 + (@CordaCenter[1] - @ClearancePoint[1])**2 ), 16), 20 )

@Raio = (@Flecha**2 + @Semi_Corda**2) / (2*@Flecha)

@CenterPoint1 = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), 0]

#Calculating angle between 2 vectors (Clearance detail)
@Vector1 = [(@CenterPoint1[0] - @ClearancePoint[0]), (@CenterPoint1[1] - @ClearancePoint[1]), 0]
@Vector2 = [(@CenterPoint1[0] - @Pointb[0]), (@CenterPoint1[1] - @Pointb[1]), 0]
@DotProduct = (@Vector1[0] * @Vector2[0]) + (@Vector1[1] * @Vector2[1])
@AngleT = Math.acos( @DotProduct / (@Raio**2) ).radians
@TotalT = @AngleT*2

(#Making clearance curve 1st side

#_________________________________________________________________________________
@TempInv1 = @Involute.entities.add_group
@Arc1 = @TempInv1.entities.add_arc( @CenterPoint1, [(@Pointb[0] - @Pointa[0]), (@Pointb[1] - @Pointa[1]), 0], [0,0,1], @Raio, ((180-@TotalT)/2 +@AngleT).degrees, ((180-@TotalT)/2 +@TotalT).degrees, @Detail )
@TempInv1.explode
#_________________________________________________________________________________
)


@Pointb = [Math.cos( @AngleOffset + (@TopThickness/2) )*@BaseRadius, -Math.sin( @AngleOffset + (@TopThickness/2) )*@BaseRadius, 0]
@Pointa = [Math.cos( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, -Math.sin( @AngleOffset + (@TopThickness/2) + @ClearanceAngle )*@BaseRadius, 0]
@CenterPoint2 = [ Math.cos(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), -Math.sin(@PitchAngle/2)*(@BaseRadius-@Clearance+@Raio), 0]

(#Making clearance curve 2nd side

#_________________________________________________________________________________
@TempInv2 = @Involute.entities.add_group
@Arc2 = @TempInv2.entities.add_arc( @CenterPoint2, [(@Pointb[0] - @Pointa[0]), (@Pointb[1] - @Pointa[1]), 0], [0,0,1], @Raio, ((180-@TotalT)/2).degrees, ((180-@TotalT)/2 +@AngleT).degrees, @Detail )
@TempInv2.explode
#_________________________________________________________________________________
)


print 'Make Spur Gear - Debug point 3/5 ok'


#Making base arc to close involute
@baseArc = @Involute.entities.add_arc [0,0,0], [1, 0, 0] , [0,0,1], @BaseRadius-@Clearance, -@PitchAngle/2, @PitchAngle/2, @Detail

#Making involute face
@InvSilhouete = @baseArc[1].all_connected
@InvFace = @Involute.entities.add_face(@InvSilhouete)
if @InvFace.normal.z == -1
@InvFace.reverse!
end
status = @InvFace.material = @GearColor
status = @InvFace.back_material = @GearColor

#Extrusion of involute face
status = @InvFace.pushpull(@Width, true)
@Involute = @Involute.to_component

#Making external gear group
@Gear = Sketchup.active_model.entities.add_group
@All_inv = @Gear.entities.add_group
@All_inv.entities.add_instance(@Involute.definition, @Involute.transformation*@All_inv.transformation)

(#Copy and rotation of teeths
 for step in 1..(@nTeeth-1)
 @Extend = @Involute.copy
 
transformation = Geom::Transformation.rotation([0,0,0], [0,0,1], @PitchAngle*step )
@Extend.transform!(transformation)

 @All_inv.entities.add_instance(@Extend.definition, @Extend.transformation*@All_inv.transformation)
 status = @Extend.erase!
 end
 status = @Involute.erase!
)

print 'Make Spur Gear - Debug point 4/5 ok'
#-------------------------------------------------------------------------------

(#Body making
Sketchup.active_model.selection.clear
model = Sketchup.active_model
entities = model.active_entities
@BaseGroup = @Gear.entities.add_group

@Bodyc1 = @BaseGroup.entities.add_circle([0,0,0], [0, 0, 1] , @BaseRadius-@Clearance, @Detail*@nTeeth)
@Bodyc2 = @BaseGroup.entities.add_circle([0,0,0], [0, 0, 1] , @HoleDiameter/2, @Detail)
@Bodyf1 = @BaseGroup.entities.add_face(@Bodyc1)
@Bodyf2 = @BaseGroup.entities.add_face(@Bodyc2)
status = @Bodyf2.erase!

if @Bodyf1.normal.z == -1
@Bodyf1.reverse!
end
status = @Bodyf1.material = @GearColor
status = @Bodyf1.back_material = @GearColor
status = @Bodyf1.pushpull(@Width, true)
)


#Make contruction references
if (@MakeRef == "Yes")
model = Sketchup.active_model
entities = model.active_entities
@Refs = @Gear.entities.add_group
@CenterPoint = @Refs.entities.add_cpoint [0, 0, @Width/2]
@RefLine = @Refs.entities.add_line [0,0,-@PitchRadius+@Width/2], [0,0,@PitchRadius+@Width/2]
@RefCircle = @Refs.entities.add_circle([0,0,@Width/2], [0, 0, 1] , @PitchRadius, @nTeeth)
end

#Reset gear for final scale
scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
@Gear.transformation *= scale_transform

#-------------------------------------------------------------------------------
print 'Make Spur Gear - Debug point 5/5 ok'


#End Message
@EndMessage = "Success. :D \n Object placed at origin[0,0,0]."
UI.messagebox(@EndMessage)
#-------------------------------------------------------------------------------

print "Success. \n \n"
end

def self.linear_spiral_dialog
#-------------------------------------------------------------------------------
prompts = ["Radius1 (base): ", "Radius2 (top): ", "Height: ", "Revolutions: ", "Edges per revolution: ", "Make tube? ", "Tube diameter: ", "Tube detail (sides): ", "Construct ref.?: "]
defaults = [ 20.00.mm, 10.00.mm, 100.00.mm, 10, 36, "Yes", 5.00.mm, 12, "Yes" ]
list = ["", "", "", "", "", "Yes|No", "", "", "Yes|No"]
input = UI.inputbox(prompts, defaults, list, "Make Linear Spring")
if input
@Radius1, @Radius2, @Height, @Revolutions, @Edges_per_revolution, @Make_Tube, @Tube_diameter, @Tube_detail, @MakeRef = input;
else
 abort("Make Linear Spring error: LSxV.000 - Operation canceled.")
end
#-------------------------------------------------------------------------------

#Timer start
@start_time = Time.now

#Log
print 'Make Linear Spring operation:'
print 'Input log: ' + input[0].to_s + ' | ' + input[1].to_s + ' | ' + input[2].to_s + ' | ' + input[3].to_s + ' | ' + input[4].to_s + ' | ' + input[5].to_s + ' | ' + input[6].to_s + ' | ' + input[7].to_s + ' | ' + input[8].to_s


(#Verifications

# Sphere radius > 0
if (@Radius1 <= 0) or (@Radius2 <= 0)
UI.messagebox("error: LSxV.001 - Both radius must be higher than 0.")
abort("Make Parts error: LSxV.001")
end

# Revolutions > 0
if (@Revolutions <= 0)
UI.messagebox("error: LSxV.002 - Revolutions must be higher than 0.")
abort("Make Parts error: LSxV.002")
end

# Edges_per_revolution > 0
if (@Edges_per_revolution <= 0)
UI.messagebox("error: LSxV.003 - Edges per revolution must be higher than 0.")
abort("Make Parts error: LSxV.003")
end

# Tube_diameter > 0
if (@Tube_diameter <= 0)
UI.messagebox("error: LSxV.004 - Tube diameter must be higher than 0.")
abort("Make Parts error: LSxV.004")
end

# Tube_diameter limit
if ( @Tube_diameter >= (@Height / @Revolutions))
UI.messagebox("error: LSxV.005 - Tube diameter is too large.")
abort("Make Parts error: LSxV.005")
end

# Tube_detail > 0
if (@Tube_detail <= 0)
UI.messagebox("error: LSxV.006 - Tube detail must be higher than 0.")
abort("Make Parts error: LSxV.006")
end
)

print 'Make Spheric Spring - Debug point 1/2 ok'

(#Camera and Boost -------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------
)

@Scale = 1000.00
@scale_transform = Geom::Transformation.scaling([0, 0, 0], 1/@Scale)

#-------------------------------------------------------------------------------
if @Radius1 == @Radius2

   print 'Make Spheric Spring - Debug point (1 radius) ok'
   #1 radius spiral
   def self.spiral1(radius, height, laps, edgesperlap, tube_diameter, tube_detail, make_reference)
   model = Sketchup.active_model
   entities = model.active_entities

   @Spiral1 = Sketchup.active_model.entities.add_group
   @Reference = @Spiral1.entities.add_group


   #Defining variables
   radius = radius.to_f * @Scale
   height = height.to_f * @Scale
   laps = laps.to_i
   edgesperlap = edgesperlap.to_i
   tube_diameter = tube_diameter.to_f * @Scale
   tube_detail = tube_detail.to_i
   @SpringColor = [40, 40, 40]

   @TotalEdges = laps * edgesperlap
   @Zincrement = height / @TotalEdges
   @EdgeAngle = 360.00.degrees / edgesperlap
   @LapHeight = height / laps

    #Make path reference
    @point1 = [radius, 0, 0]
    for step in 0..@TotalEdges
     @x = Math.cos(@EdgeAngle * step) * radius
     @y = Math.sin(@EdgeAngle * step) * radius
     @z = step * @Zincrement
     @point2 = [@x, @y, @z]
     @line = @Reference.entities.add_line @point1, @point2
     @point1 = @point2
    end
    
     #Get edge and path reference
   @RefEdge = @Reference.entities[0]
   @Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
   @Path = @RefEdge.all_connected
   
   if (@Make_Tube == "Yes")
    @Tube = @Spiral1.entities.add_group
   #Make tube face
   @C1 = @Tube.entities.add_circle(@RefEdge.start.position, @Vector , tube_diameter/2, tube_detail)
   @F1 = @Tube.entities.add_face(@C1)

   #-------------------------------------------------------------------------------
   #Set face direction
   @D = 3
   @VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
   @UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

   @FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
   @FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

   if @UnitVector == @FaceUnitVector
   @F1.reverse!
   end
   #-------------------------------------------------------------------------------

   #Apply face color
   @F1.material = @SpringColor
   @F1.back_material = @SpringColor

   #Make tube
   @Structure = @F1.followme(@Path)
   @Tube.transformation *= @scale_transform
   
   @Reference.erase!
   end
   
   
    if make_reference == "Yes"
     
     if (@Make_Tube == "Yes")
      @Reference = @Spiral1.entities.add_group
     end
     
     @Pt1 = @Reference.entities.add_cpoint [0, 0, 0]
     @Pt2 = @Reference.entities.add_cpoint [0, 0, height]
     @Cp = @Reference.entities.add_cpoint [0, 0, height/2]
     @Rl = @Reference.entities.add_line [0, 0, 0], [0, 0, height]
     @Reference.transformation *= @scale_transform    
    else
    
     if (@Make_Tube == "No")
      @Reference.transformation *= @scale_transform
     end    
    end


   end

   Tax_Stuffs.spiral1(@Radius1, @Height, @Revolutions, @Edges_per_revolution, @Tube_diameter, @Tube_detail, @MakeRef)

else #--------------------------------------------------------------------------

   print 'Make Spheric Spring - Debug point (2 radius) ok'
   #2 radius spiral
   def self.spiral2(radius1, radius2, height, laps, edgesperlap, tube_diameter, tube_detail, make_reference)
   model = Sketchup.active_model
   entities = model.active_entities

   @Spiral2 = Sketchup.active_model.entities.add_group
   @Reference = @Spiral2.entities.add_group


   #Defining variables
   radius1 = (radius1 * @Scale).to_f
   radius2 = (radius2 * @Scale).to_f
   height = (height * @Scale).to_f
   laps = laps.to_i
   edgesperlap = edgesperlap.to_i
   tube_diameter = (tube_diameter * @Scale).to_f
   tube_detail = tube_detail.to_i
   @SpringColor = [40, 40, 40]

   @TotalEdges = laps * edgesperlap
   @Zincrement = height / @TotalEdges
   @EdgeAngle = 360.00.degrees / edgesperlap
   @LapHeight = height / laps

   @Reduction = radius2 - radius1
   @ReductionPerEdge = @Reduction / @TotalEdges

    @point1 = [radius1, 0, 0]
    for step in 0..@TotalEdges
     @Radius = (radius1 + (@ReductionPerEdge * step))
     @x = Math.cos(@EdgeAngle * step) * @Radius
     @y = Math.sin(@EdgeAngle * step) * @Radius
     @z = step * @Zincrement
     @point2 = [@x, @y, @z]
     @line = @Reference.entities.add_line @point1, @point2
     @point1 = @point2
    end
    
    #Get edge and path reference
   @RefEdge = @Reference.entities[0]
   @Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
   @Path = @RefEdge.all_connected
   
   if (@Make_Tube == "Yes")
    @Tube = @Spiral2.entities.add_group
   #Make tube face
   @C1 = @Tube.entities.add_circle(@RefEdge.start.position, @Vector , tube_diameter/2, tube_detail)
   @F1 = @Tube.entities.add_face(@C1)

   #-------------------------------------------------------------------------------
   #Set face direction
   @D = 3
   @VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
   @UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

   @FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
   @FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

   if @UnitVector == @FaceUnitVector
   @F1.reverse!
   end
   #-------------------------------------------------------------------------------

   #Apply face color
   @F1.material = @SpringColor
   @F1.back_material = @SpringColor

   #Make tube
   @Structure = @F1.followme(@Path)
   @Tube.transformation *= @scale_transform
   
   @Reference.erase!
   end
   
   
    if make_reference == "Yes"
     
     if (@Make_Tube == "Yes")
      @Reference = @Spiral1.entities.add_group
     end
     
     @Pt1 = @Reference.entities.add_cpoint [0, 0, 0]
     @Pt2 = @Reference.entities.add_cpoint [0, 0, height]
     @Cp = @Reference.entities.add_cpoint [0, 0, height/2]
     @Rl = @Reference.entities.add_line [0, 0, 0], [0, 0, height]
     @Reference.transformation *= @scale_transform    
    else
    
     if (@Make_Tube == "No")
      @Reference.transformation *= @scale_transform
     end    
    end
   
   end

   Tax_Stuffs.spiral2(@Radius1, @Radius2, @Height, @Revolutions, @Edges_per_revolution, @Tube_diameter, @Tube_detail, @MakeRef)

end

#-------------------------------------------------------------------------------
print 'Make Spheric Spring - Debug point 2/2 ok'
 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
#-------------------------------------------------------------------------------

end

def self.spheric_spiral_dialog
#-------------------------------------------------------------------------------
prompts = ["Sphere radius: ", "Revolutions: ", "Edges per revolution: ", "Cutoff (%): ", "Stretch factor: ", "Tube diameter: ", "Tube detail (sides): ", "Construct ref.?: "]
defaults = [50.00.mm, 10, 24, 5, 1, 5.00.mm, 12, "Yes"]
list = ["", "", "", "", "", "", "", "Yes|No"]
val = UI.inputbox(prompts, defaults, list, "Make Spheric Spring")
if val
@Sphere_radius, @Revolutions, @Edges_per_revolution, @Cutoff, @Stretch_factor, @Tube_diameter, @Tube_detail, @MakeRef = val;
else
 abort("Make Spheric Spring error: SSxV.000 - Operation canceled.")
end
#-------------------------------------------------------------------------------

#Timer start
@start_time = Time.now

#Log and verification
print 'Make Spheric Spring operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s


(#Verifications

# Sphere radius > 0
if (@Sphere_radius <= 0)
UI.messagebox("error: SSxV.001 - Sphere radius must be higher than 0.")
abort("Make Parts error: SSxV.001")
end

# Revolutions > 0
if (@Revolutions <= 0)
UI.messagebox("error: SSxV.002 - Revolutions must be higher than 0.")
abort("Make Parts error: SSxV.002")
end

# Edges per rev > 0
if (@Edges_per_revolution <= 0)
UI.messagebox("error: SSxV.003 - Edges per revolution must be higher than 0.")
abort("Make Parts error: SSxV.003")
end

# Cutoff 0 > c < 100
if (@Cutoff <= 0) or (@Cutoff >= 100)
UI.messagebox("error: SSxV.004 - Cutoff must be in range (0 > C < 100).")
abort("Make Parts error: SSxV.004")
end

# Strech > 0
if (@Stretch_factor <= 0)
UI.messagebox("error: SSxV.005 - Stretch factor must be higher than 0.")
abort("Make Parts error: SSxV.005")
end

# Tube radius range
if (@Tube_diameter <= 0)
UI.messagebox("error: SSxV.006 - Tube diameter must be higher than 0.")
abort("Make Parts error: SSxV.006")
end

# Tube radius range
if ( (@Revolutions * @Tube_diameter /4) >= (@Sphere_radius * @Stretch_factor) )
UI.messagebox("error: SSxV.007 - Tube diameter is too large.")
abort("Make Parts error: SSxV.007")
end

# Tube detail > 0
if (@Tube_detail <= 0)
UI.messagebox("error: SSxV.008 - Tube detail must be higher than 0.")
abort("Make Parts error: SSxV.008")
end
)

print 'Make Spheric Spring - Debug point 1/5 ok'

(#-------------------------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([2, -2, 2], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------
)


#Make Spheric Spring
def self.spiral_spring(radius, revolutions, edgesperrev, cutoff, stretchfactor, tubediameter, detail, makereferences)
#Defining group
model = Sketchup.active_model
entities = model.active_entities
@Spring = Sketchup.active_model.entities.add_group
@Tube = @Spring.entities.add_group
@Reference = @Spring.entities.add_group

#Defining variables
@Scale = 10000.00
radius = (radius * @Scale).to_f
revolutions = revolutions.to_i
edgesperrev = edgesperrev.to_i
cutoff = cutoff.to_i
tubediameter = (tubediameter * @Scale).to_f
detail = detail.to_i
@SpringColor = [40, 40, 40]


@TotalEdges = revolutions * edgesperrev
@VerticalStep = 180.00.degrees / @TotalEdges
@HorizontalStep = 360.00.degrees / edgesperrev

#CutOff (0% - 100%)
@cut = (@TotalEdges / 100.00 * cutoff)
@start = (@cut).to_i
@ending = (@TotalEdges - @cut).to_i

print 'Make Spheric Spring - Debug point 2/5 ok'

#Defining 1st point
@LocalRadius = Math.sin(@VerticalStep * @start) * radius
@x = Math.cos(@HorizontalStep * @start) * @LocalRadius
@y = Math.sin(@HorizontalStep * @start) * @LocalRadius
@z = Math.cos(@VerticalStep * @start) * radius * stretchfactor
@point1 = [@x, @y, @z]

@PointRef = @point1

#Making spiral
 for edge in @start+1..@ending
 @LocalRadius = Math.sin(@VerticalStep * edge) * radius

 @x = Math.cos(@HorizontalStep * edge) * @LocalRadius
 @y = Math.sin(@HorizontalStep * edge) * @LocalRadius
 @z = Math.cos(@VerticalStep * edge) * radius * stretchfactor

 @point2 = [@x, @y, @z]
 @line = @Reference.entities.add_line @point1, @point2
 @point1 = @point2
 end

print 'Make Spheric Spring - Debug point 3/5 ok'

#Get edge and path reference
@RefEdge = @Reference.entities[0]
@Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
@Path = @RefEdge.all_connected

#Make tube face
@C1 = @Tube.entities.add_circle(@PointRef, @Vector , tubediameter/2, detail)
@F1 = @Tube.entities.add_face(@C1)

#Set face direction
@D = 3
@VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
@UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

@FaceVectorModule = ((@F1.normal.x**2) + (@F1.normal.y**2) + (@F1.normal.z**2))**0.5
@FaceUnitVector = [ (@F1.normal.x/@FaceVectorModule).round(@D), (@F1.normal.y/@FaceVectorModule).round(@D), (@F1.normal.z/@FaceVectorModule).round(@D) ]

if @UnitVector == @FaceUnitVector
@F1.reverse!
end

print 'Make Spheric Spring - Debug point 4/5 ok'

#Apply face color
@F1.material = @SpringColor
@F1.back_material = @SpringColor

#Make tube
@Structure = @F1.followme(@Path)

if makereferences == "No"
@Reference.erase!
else
@CenterPoint = @Reference.entities.add_cpoint [0, 0, 0]
end

scale_transform = Geom::Transformation.scaling([0, 0, 0], 1/@Scale)
@Spring.transformation *= scale_transform
end

Tax_Stuffs.spiral_spring(@Sphere_radius, @Revolutions, @Edges_per_revolution, @Cutoff, @Stretch_factor, @Tube_diameter, @Tube_detail, @MakeRef)

print 'Make Spheric Spring - Debug point 5/5 ok'
#-------------------------------------------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
#-------------------------------------------------------------------------------
end

def self.adjustable_suspension_dialog
#-------------------------------------------------------------------------------
@Detail = 24
@Spring_Revolution_Detail = 24
@Spring_Tube_Detail = 12
@Suspension_Length = 500.00.mm
@Suspension_Extension = 50.00
@Spring_Compression = 10.00
@Cilinder_Diameter = 20.00.mm
@Casing_Thickness = 25.00.mm
@Casing_Screw_Laps = 50
@Plate_Thickness = 20.00.mm
@Compressor_Thickness = 25.00.mm
@Hole_Diameter = 20.00.mm
@End_Elongation = 15.00.mm
@Anchor_Thickness = 15.00.mm
@Spring_Revolutions = 10
@Spring_Tube_Diameter = 15.00.mm
@MakeRef = "No"

@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["General detail: ", "Spring revolution detail: ", "Spring tube detail: ", "Suspension length: ", "Suspension extension (0-100)%: ", "Spring compression (0-95)%: ", "Cilinder diameter: ", "Casing thickness: ", "Casing screw laps: ", "Plate thickness: ", "Compressor thickness: ", "Hole diameter: ", "End elongation: ", "Anchor thickness: ", "Spring revolutions: ", "Spring tube diameter: ", "Make references: "]
defaults = [@Detail, @Spring_Revolution_Detail, @Spring_Tube_Detail, @Suspension_Length, @Suspension_Extension, @Spring_Compression, @Cilinder_Diameter, @Casing_Thickness, @Casing_Screw_Laps, @Plate_Thickness, @Compressor_Thickness, @Hole_Diameter, @End_Elongation, @Anchor_Thickness, @Spring_Revolutions, @Spring_Tube_Diameter, @MakeRef]
list = ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "Yes|No"]

val = UI.inputbox(prompts, defaults, list, "Make Adjustable Suspension")
if val
@Detail, @Spring_Revolution_Detail, @Spring_Tube_Detail, @Suspension_Length, @Suspension_Extension, @Spring_Compression, @Cilinder_Diameter, @Casing_Thickness, @Casing_Screw_Laps, @Plate_Thickness, @Compressor_Thickness, @Hole_Diameter, @End_Elongation, @Anchor_Thickness, @Spring_Revolutions, @Spring_Tube_Diameter, @MakeRef = val;
else
 abort("Make Adjustable Suspension error: ASxV.000 - Operation canceled.")
end
#-------------------------------------------------------------------------------

#Log and verification
print 'Make Adjustable Suspension operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s + ' | ' + val[8].to_s + ' | ' + val[9].to_s + ' | ' + val[10].to_s + ' | ' + val[11].to_s + ' | ' + val[12].to_s + ' | ' + val[13].to_s + ' | ' + val[14].to_s + ' | ' + val[15].to_s + ' | ' + val[16].to_s

print 'Make Adjustable Suspension - Debug point 1/7 ok'

(#VERIFICATIONS


#Detail < 6
if (@Detail < 6)
UI.messagebox("error: ASxV.001 - Min general detail is 6.")
warn("Make Parts error: ASxV.001")
@Verification = "NOT OK"
end

#Detail < 6
if (@Spring_Revolution_Detail < 6)
UI.messagebox("error: ASxV.002 - Min spring revolution detail is 6.")
warn("Make Parts error: ASxV.002")
@Verification = "NOT OK"
end

#Detail < 6
if (@Spring_Tube_Detail < 6)
UI.messagebox("error: ASxV.003 - Min spring tube detail is 6.")
warn("Make Parts error: ASxV.003")
@Verification = "NOT OK"
end

#Suspension_Length > 0
if (@Suspension_Length <= 0)
UI.messagebox("error: ASxV.004 - Suspension length must be higher than 0.")
warn("Make Parts error: ASxV.004")
@Verification = "NOT OK"
end

#0 < Suspension_Extension > 100
if (@Suspension_Extension < 0) or (@Suspension_Extension > 100)
UI.messagebox("error: ASxV.005 - Suspension extension must be in range (0-100).")
warn("Make Parts error: ASxV.005")
@Verification = "NOT OK"
end

#0 < Spring_Compression > 95
if (@Spring_Compression < 0) or (@Spring_Compression > 95)
UI.messagebox("error: ASxV.006- Suspension extension must be in range (0-95).")
warn("Make Parts error: ASxV.006")
@Verification = "NOT OK"
end

#Cilinder_Diameter > 0
if (@Cilinder_Diameter <= 0)
UI.messagebox("error: ASxV.007 - Cilinder diameter must be higher than 0.")
warn("Make Parts error: ASxV.007")
@Verification = "NOT OK"
end

#Casing_Thickness > 0
if (@Casing_Thickness <= 0)
UI.messagebox("error: ASxV.008 - Casing thickness must be higher than 0.")
warn("Make Parts error: ASxV.008")
@Verification = "NOT OK"
end

#@Plate_Thickness limit
if (@Plate_Thickness <= 0)
UI.messagebox("error: ASxV.009 - Plate thickness must be higher than 0.")
warn("Make Parts error: ASxV.009")
@Verification = "NOT OK"
end

#@Plate_Thickness limit
if (@Plate_Thickness > @Suspension_Length/4)
UI.messagebox("error: ASxV.010 - Plate thickness is too high.")
warn("Make Parts error: ASxV.010")
@Verification = "NOT OK"
end

#@Compressor_Thickness limit
if (@Compressor_Thickness <= 0)
UI.messagebox("error: ASxV.0011 - Compressor thickness must be higher than 0.")
warn("Make Parts error: ASxV.0011")
@Verification = "NOT OK"
end

#@Compressor_Thickness limit
if (@Compressor_Thickness > @Suspension_Length/4)
UI.messagebox("error: ASxV.012 - Compressor thickness is too high.")
warn("Make Parts error: ASxV.012")
@Verification = "NOT OK"
end

#@End_Elongation limit
if (@End_Elongation < 0)
UI.messagebox("error: ASxV.013 - End elongation can't be negative.")
warn("Make Parts error: ASxV.013")
@Verification = "NOT OK"
end

#@End_Elongation limit
if (@End_Elongation > @Suspension_Length/4)
UI.messagebox("error: ASxV.014 - End elongation is too high.")
warn("Make Parts error: ASxV.014")
@Verification = "NOT OK"
end

#@Hole_Diameter min
if (@Hole_Diameter <= 0)
UI.messagebox("error: ASxV.015 - Hole diameter must be higher than 0.")
warn("Make Parts error: ASxV.015")
@Verification = "NOT OK"
end

#@Anchor_Thickness min
if (@Anchor_Thickness <= 0)
UI.messagebox("error: ASxV.016 - Anchor thickness must be higher than 0.")
warn("Make Parts error: ASxV.016")
@Verification = "NOT OK"
end

#@Hole_Diameter limit
if (@Hole_Diameter >= (@Cilinder_Diameter + 2*@Casing_Thickness) )
UI.messagebox("error: ASxV.017 - Hole diameter or Anchor thickness is too large.")
warn("Make Parts error: ASxV.017")
@Verification = "NOT OK"
end

#Casing_Screw_Laps min
if (@Casing_Screw_Laps <= 0)
UI.messagebox("error: ASxV.018 - Casing screw laps must be higher than 0.")
warn("Make Parts error: ASxV.018")
@Verification = "NOT OK"
end

#@Spring_Revolutions min
if (@Spring_Revolutions <= 0)
UI.messagebox("error: ASxV.019 - Spring revolutions must be higher than 0.")
warn("Make Parts error: ASxV.019")
@Verification = "NOT OK"
end

#@Spring_Tube_Diameter min
if (@Spring_Tube_Diameter <= 0)
UI.messagebox("error: ASxV.020 - Spring tube diameter must be higher than 0.")
warn("Make Parts error: ASxV.020")
@Verification = "NOT OK"
end
)

end

#-------------------------------------------------------------------------------
print 'Make Adjustable Suspension - Debug point 2/7 ok'
UI.messagebox("It can take a few minutes, please wait.")
#Timer start
@start_time = Time.now

(#CAMERA AND BOOST --------------------------------------------------------------------

#Define camera view
camera = Sketchup::Camera.new([15, -15, 15], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)

#-------------------------------------------------------------------------------
)

(#Suspension --------------------------------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities
@Suspension = Sketchup.active_model.entities.add_group
@Structure = @Suspension.entities.add_group
@Spring = @Suspension.entities.add_group
@Reference = @Suspension.entities.add_group


#Generic Variables
@Scale = 10000.00

@Spring_Color = [200, 40, 40]
@Cilinder_Color = [178,191,209]
@Casing_Color = [30,30,30]
@Compressor_Color = [100,20,20]
@End_Color = [80,80,80]


#Suspension Variables
@Suspension_Length = (@Suspension_Length * @Scale).to_f
@Suspension_Extension = (@Suspension_Extension).to_f
@Spring_Compression = (@Spring_Compression).to_f

@Cilinder_Diameter = (@Cilinder_Diameter * @Scale).to_f
@Casing_Thickness = (@Casing_Thickness * @Scale).to_f
@Casing_Screw_Laps = (@Casing_Screw_Laps).to_i
@Plates_Radius = 0.00
@Plate_Thickness = (@Plate_Thickness * @Scale).to_f
@Compressor_Thickness = (@Compressor_Thickness * @Scale).to_f
@Anchor_Thickness = (@Anchor_Thickness * @Scale).to_f
@Hole_Diameter = (@Hole_Diameter * @Scale).to_f
@End_Elongation = (@End_Elongation * @Scale).to_f


#Spring Variables
@Spring_Radius = 0.00
@Spring_Length = 0.00

@Spring_Revolutions = (@Spring_Revolutions).to_i
@Spring_Revolution_Detail = (@Spring_Revolution_Detail).to_i

@Spring_Tube_Diameter = (@Spring_Tube_Diameter * @Scale).to_f
@Spring_Tube_Detail = (@Spring_Tube_Detail).to_i

@MakeRef = @MakeRef.to_s
@Spring_Reference = "No"
#--------------------------------------------------------------------------
)

(#Initial MATH
@Inside_Range = @Suspension_Length - (@Hole_Diameter + 2*@Anchor_Thickness + 2*@End_Elongation + 2*@Plate_Thickness)
@Half_Range = @Inside_Range/2

@Cilinder_Length = ((1.00 - (@Suspension_Extension/100.00)) * @Half_Range) +@Half_Range
@Casing_Length = @Cilinder_Length

@Compression_Range = (1.00 - (@Spring_Compression/100.00)) * (@Casing_Length - @Compressor_Thickness)
@Spring_Length = @Inside_Range - (@Casing_Length - @Compression_Range)

#VERIFICATION ---------------------------------- MAX @Spring_Tube_Diameter
if (@Spring_Tube_Diameter > (@Spring_Length / @Spring_Revolutions)/2 )
UI.messagebox("error: ASxV.021 - Spring tube diameter is too large.")
abort("Make Parts error: ASxV.021")
end
#-------------------------------------------------------------------------

@Casing_Out_Radius = (@Cilinder_Diameter/2) + @Casing_Thickness
@Plates_Radius = @Casing_Out_Radius + (@Spring_Tube_Diameter * 1.5)

@Spring_Radius = @Casing_Out_Radius + (@Spring_Tube_Diameter/2)
#--------------------------------------------------------------------------
)

(#References
@Half_Susp_Length = (@Suspension_Length/2)
@Center_Point = @Reference.entities.add_cpoint [0,0,0]
@Top_Hole_Center = @Reference.entities.add_cpoint [0,0,@Half_Susp_Length]
@Bottom_Hole_Center = @Reference.entities.add_cpoint [0,0,-@Half_Susp_Length]

@Length_Line = @Reference.entities.add_line [0, 0, @Half_Susp_Length], [0, 0, -@Half_Susp_Length]
@Anchor_Line1 = @Reference.entities.add_line [0, @Half_Susp_Length, @Half_Susp_Length], [0, -@Half_Susp_Length, @Half_Susp_Length]
@Anchor_Line2 = @Reference.entities.add_line [0, @Half_Susp_Length, -@Half_Susp_Length], [0, -@Half_Susp_Length, -@Half_Susp_Length]

@Ref_Top_Hole = @Reference.entities.add_circle([0,0,@Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2, @Detail)
@Ref_Bottom_Hole = @Reference.entities.add_circle([0,0,-@Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2, @Detail)

@Plate_Offset = (@Hole_Diameter/2 + @Anchor_Thickness + @End_Elongation)
@Ref_Top_Plate = @Reference.entities.add_circle([0,0, (@Half_Susp_Length-@Plate_Offset)], [0, 0, 1] , @Plates_Radius, @Detail)
@Ref_Bottom_Plate = @Reference.entities.add_circle([0,0, -(@Half_Susp_Length-@Plate_Offset)], [0, 0, 1] , @Plates_Radius, @Detail)

@Ref_Cilinder1 = @Reference.entities.add_circle([0,0, @Half_Range], [0, 0, 1] , @Cilinder_Diameter/2, @Detail)
@Ref_Cilinder2 = @Reference.entities.add_circle([0,0, @Half_Range-@Cilinder_Length], [0, 0, 1] , @Cilinder_Diameter/2, @Detail)

@Ref_Casing1 = @Reference.entities.add_circle([0,0, -@Half_Range], [0, 0, 1] , @Casing_Out_Radius, @Detail)
@Ref_Casing2 = @Reference.entities.add_circle([0,0, (-@Half_Range+@Casing_Length)], [0, 0, 1] , @Casing_Out_Radius, @Detail)

@Ref_Compressor = @Reference.entities.add_circle([0,0, (-@Half_Range+@Casing_Length-@Compression_Range)], [0, 0, 1] , @Plates_Radius, @Detail)

@Spring_ZPosition = @Half_Range - (@Spring_Length/2)
@Spring_Center_Point = @Reference.entities.add_cpoint [0,0,@Spring_ZPosition]
@Spring_Ref_Radius = @Reference.entities.add_circle([0,0,@Spring_ZPosition], [0, 0, 1] , @Spring_Radius, @Detail)
#--------------------------------------------------------------------------

#Spring Reference
@SReference = @Reference.entities.add_group

#Defining parameters
@TotalEdges = @Spring_Revolutions * @Spring_Revolution_Detail
@Zincrement = @Spring_Length / @TotalEdges #Pico = media * pi/2

@ZincrementMed = @Spring_Length / @TotalEdges
@ZincrementMax = @ZincrementMed * (Math::PI/2)
@ParaboleUnit = (180.00 / @TotalEdges).degrees

@EdgeAngle = 360.00.degrees / @Spring_Revolution_Detail
@LapHeight = @Spring_Length / @Spring_Revolutions
@Spring_Start_Height = (-@Half_Range+@Casing_Length-@Compression_Range)

 #Make path reference
 @point1 = [@Spring_Radius, 0, @Spring_Start_Height]
 for step in 0..@TotalEdges
  @x = Math.cos(@EdgeAngle * step) * @Spring_Radius
  @y = Math.sin(@EdgeAngle * step) * @Spring_Radius

  @Zincrement = Math.sin(step * @ParaboleUnit) * @ZincrementMax
  @z = @point1.z + @Zincrement

  @point2 = [@x, @y, @z]
  @line = @SReference.entities.add_line @point1, @point2
  @point1 = @point2
 end
#--------------------------------------------------------------------------
)

print 'Make Adjustable Suspension - Debug point 3/7 ok'

(#Make spring tube

#Get edge and path reference
@RefEdge = @SReference.entities[0]
@Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
@Path = @RefEdge.all_connected

#Make tube face
@Spring_Tube_Circle = @Spring.entities.add_circle(@RefEdge.start.position, @Vector , @Spring_Tube_Diameter/2, @Spring_Tube_Detail)
@Spring_Tube_Face = @Spring.entities.add_face(@Spring_Tube_Circle)

#-------------------------------------------------------------------------------

#Set face direction
@D = 3
@VectorModule = ((@Vector.x**2) + (@Vector.y**2) + (@Vector.z**2))**0.5
@UnitVector = [ (@Vector.x/@VectorModule).round(@D), (@Vector.y/@VectorModule).round(@D), (@Vector.z/@VectorModule).round(@D) ]

@FaceVectorModule = ((@Spring_Tube_Face.normal.x**2) + (@Spring_Tube_Face.normal.y**2) + (@Spring_Tube_Face.normal.z**2))**0.5
@FaceUnitVector = [ (@Spring_Tube_Face.normal.x/@FaceVectorModule).round(@D), (@Spring_Tube_Face.normal.y/@FaceVectorModule).round(@D), (@Spring_Tube_Face.normal.z/@FaceVectorModule).round(@D) ]

if @UnitVector == @FaceUnitVector
@Spring_Tube_Face.reverse!
end

#-------------------------------------------------------------------------------

#Apply face color
@Spring_Tube_Face.material = @Spring_Color
@Spring_Tube_Face.back_material = @Spring_Color

#Make tube
@Spring = @Spring_Tube_Face.followme(@Path)
)

#--------------------------------------------------------------------------------------

#MODELING
model = Sketchup.active_model
entities = model.active_entities

(# CILINDER----------------------------------------------------------------------
@Cilinder = @Structure.entities.add_group

@Cilinder_Circle = @Cilinder.entities.add_circle([0,0, @Half_Range], [0, 0, 1] , @Cilinder_Diameter/2, @Detail)
@Cilinder_Face = @Cilinder.entities.add_face(@Cilinder_Circle)
if @Cilinder_Face.normal.z == 1
@Cilinder_Face.reverse!
end

#Apply face color
@Cilinder_Face.material = @Cilinder_Color
@Cilinder_Face.back_material = @Cilinder_Color

@Cilinder_Face.pushpull(@Cilinder_Length, true)
)

print 'Make Adjustable Suspension - Debug point 4/7 ok'

(# CASING------------------------------------------------------------------------
@Casing = @Structure.entities.add_group
@Casing_Bevel = @Casing_Thickness/4

@Casing_Circle1 = @Casing.entities.add_circle([0,0, -@Half_Range], [0, 0, 1] , @Cilinder_Diameter/2, @Detail)
@Casing_Circle2 = @Casing.entities.add_circle([0,0, -@Half_Range], [0, 0, 1] , @Casing_Out_Radius, @Detail)

@Casing_Face1 = @Casing.entities.add_face(@Casing_Circle1)
@Casing_Face2 = @Casing.entities.add_face(@Casing_Circle2)
@Casing_Face1.erase!

if @Casing_Face2.normal.z == -1
@Casing_Face2.reverse!
end

#Apply face color
@Casing_Face2.material = @Casing_Color
@Casing_Face2.back_material = @Casing_Color

@Casing_Face2.pushpull(@Casing_Length-@Casing_Bevel, true)

@Casing_Circle3 = @Casing.entities.add_circle([0,0, -@Half_Range+@Casing_Length-@Casing_Bevel], [0, 0, 1] , @Cilinder_Diameter/2, @Detail)
@Casing_Circle4 = @Casing.entities.add_circle([0,0, -@Half_Range+@Casing_Length-@Casing_Bevel], [0, 0, 1] , @Cilinder_Diameter/2+@Casing_Bevel, @Detail)
@Casing_Circle42 = @Casing.entities.add_circle([0,0, -@Half_Range+@Casing_Length-@Casing_Bevel], [0, 0, 1] , @Cilinder_Diameter/2+(@Casing_Bevel*1.2), @Detail)

@Casing_Circle5 = @Casing.entities.add_circle([0,0, -@Half_Range+@Casing_Length-@Casing_Bevel], [0, 0, 1] , @Casing_Out_Radius, @Detail)
@Casing_Circle6 = @Casing.entities.add_circle([0,0, -@Half_Range+@Casing_Length-@Casing_Bevel], [0, 0, 1] , @Casing_Out_Radius-@Casing_Bevel, @Detail)


Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Casing_Circle3)
Sketchup.active_model.selection.add(@Casing_Circle4)

transform = Geom::Transformation.new([ 0, 0, @Casing_Bevel ])
@Casing.entities.transform_entities(transform, @Casing_Circle3)
@Casing.entities.transform_entities(transform, @Casing_Circle4)


Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Casing_Circle5)

transform = Geom::Transformation.new([ 0, 0, -@Casing_Bevel ])
@Casing.entities.transform_entities(transform, @Casing_Circle5)

@Casing.material = @Casing_Color



#--------------------------------------------------------------------------
#Screw_to_Subtract Reference

model = Sketchup.active_model
entities = model.active_entities
@Screw_to_Subtract_Path = @Structure.entities.add_group
@Screw_to_Subtract_Tube = @Structure.entities.add_group

   #Defining variables
   @Initial_Height = -@Half_Range
   @TotalEdges = @Casing_Screw_Laps * @Detail
   @Zincrement = @Casing_Length / @TotalEdges
   @EdgeAngle = 360.00.degrees / @Detail
   @Lap_Height = @Casing_Length / @Casing_Screw_Laps

    #Make path reference
    @point1 = [@Casing_Out_Radius, 0, @Initial_Height]
    for step in 0..@TotalEdges
     @x = Math.cos(@EdgeAngle * step) * @Casing_Out_Radius
     @y = Math.sin(@EdgeAngle * step) * @Casing_Out_Radius
     @z = @Initial_Height + (step * @Zincrement)
     @point2 = [@x, @y, @z]
     @line = @Screw_to_Subtract_Path.entities.add_line @point1, @point2
     @point1 = @point2
    end
    
   #Get edge and path reference
   @RefEdge = @Screw_to_Subtract_Path.entities[0]
   @Vector = @RefEdge.start.position.vector_to(@RefEdge.end.position)
   @Path = @RefEdge.all_connected

   #Make tube face
   if ( (@Lap_Height/4) > (@Casing_Thickness/3) )
    @Screw_Tube_Radius = (@Casing_Thickness/3)
   else
    @Screw_Tube_Radius = (@Lap_Height/4)
   end 
   
   @Screw_Detail = (@Detail/12) + 3
   @C1 = @Screw_to_Subtract_Tube.entities.add_circle(@RefEdge.start.position, @Vector , @Screw_Tube_Radius, @Screw_Detail.to_i)
   @F1 = @Screw_to_Subtract_Tube.entities.add_face(@C1)
   @F1.followme(@Path)
   @Screw_to_Subtract_Path.erase!
   
   @Casing = @Screw_to_Subtract_Tube.subtract(@Casing)
   
   @Casing.material = @Casing_Color
#--------------------------------------------------------------------------

)

print 'Make Adjustable Suspension - Debug point 5/7 ok'

(# ENDS--------------------------------------------------------------------------
@End = @Structure.entities.add_group
@Plate = @End.entities.add_group

#PLATE
@Plate_Circle = @Plate.entities.add_circle([0,0, (@Half_Susp_Length-@Plate_Offset)], [0, 0, 1] , @Plates_Radius, @Detail)
@Plate_Face = @Plate.entities.add_face(@Plate_Circle)
if @Plate_Face.normal.z == 1
@Plate_Face.reverse!
end

#Apply face color
@Plate_Face.material = @End_Color
@Plate_Face.back_material = @End_Color

@Plate_Face.pushpull(@Plate_Thickness, true)

#Plate detail
@Plate_Circle2 = @Plate.entities.add_circle([0,0, (@Half_Susp_Length-@Plate_Offset) ], [0, 0, 1] , @Plates_Radius-(@Plate_Thickness/2), @Detail)

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Plate_Circle)

transform = Geom::Transformation.new([ 0, 0, -(@Plate_Thickness/4)*3 ])
@Plate.entities.transform_entities(transform, @Plate_Circle)
# --------------------------------------------------------------------------

#ANCHOR
@Anchor = @End.entities.add_group
@Anc_Bev = (@Anchor_Thickness/4)
@Anc_Depth = @Cilinder_Diameter

@Anchor_Arc = @Anchor.entities.add_arc [0, 0, @Half_Susp_Length], [-1,0,0], [0,1,0], @Hole_Diameter/2+@Anchor_Thickness, 0.degrees, 180.degrees, @Detail/2

@Total_Elongation = @Hole_Diameter/2 + @Anchor_Thickness + @End_Elongation

@Elongation_Line1 = @Anchor.entities.add_line [(@Hole_Diameter/2+@Anchor_Thickness), 0, @Half_Susp_Length], [@Hole_Diameter/2+@Anchor_Thickness, 0, @Half_Susp_Length-@Total_Elongation]
@Elongation_Line2 = @Anchor.entities.add_line [-(@Hole_Diameter/2+@Anchor_Thickness), 0, @Half_Susp_Length], [-(@Hole_Diameter/2+@Anchor_Thickness), 0, @Half_Susp_Length-@Total_Elongation]

model = Sketchup.active_model
entities = model.active_entities
Sketchup.active_model.selection.clear
@Anchor_Bevel_Path1 = @Anchor_Arc[0].all_connected

@Elongation_Line3 = @Anchor.entities.add_line [-(@Hole_Diameter/2+@Anchor_Thickness), 0, @Half_Susp_Length-@Total_Elongation], [(@Hole_Diameter/2+@Anchor_Thickness), 0, @Half_Susp_Length-@Total_Elongation]

@End_Hole1 = @Anchor.entities.add_circle([0,0,@Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2, @Detail)

#Anchor Face
@Anchor_Face1 = @Anchor.entities.add_face(@End_Hole1)
@Anchor_Path = @Elongation_Line3.all_connected
@Anchor_Face2 = @Anchor.entities.add_face(@Anchor_Path)
@Anchor_Face1.erase!

if @Anchor_Face2.normal.y == -1
@Anchor_Face2.reverse!
end

#Apply face color
@Anchor_Face2.material = @End_Color
@Anchor_Face2.back_material = @End_Color

status = @Anchor_Face2.pushpull(@Anc_Depth, true)

#Bevel edges 1st side
@End_Hole2 = @Anchor.entities.add_circle([0,0,@Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2+@Anc_Bev, @Detail)
@Anchor_Arc2 = @Anchor.entities.add_arc [0, 0, @Half_Susp_Length], [-1,0,0], [0,1,0], @Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev, 0.degrees, 180.degrees, @Detail/2
@Elongation_Line12 = @Anchor.entities.add_line [(@Hole_Diameter/2+@Anchor_Thickness)-@Anc_Bev, 0, @Half_Susp_Length], [@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev, 0, @Half_Susp_Length-@Total_Elongation]
@Elongation_Line22 = @Anchor.entities.add_line [-(@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev), 0, @Half_Susp_Length], [-(@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev), 0, @Half_Susp_Length-@Total_Elongation]

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Anchor_Bevel_Path1)
transform = Geom::Transformation.new([ 0, @Anc_Bev, 0 ])
@Anchor.entities.transform_entities(transform, @Anchor_Bevel_Path1)
@Anchor.entities.transform_entities(transform, @End_Hole1)


#Bevel edges 2nd side
@End_Hole1_2 = @Anchor.entities.add_circle([0, @Anc_Depth, @Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2+@Anc_Bev, @Detail)
@Anchor_Arc1_2 = @Anchor.entities.add_arc [0, @Anc_Depth, @Half_Susp_Length], [-1,0,0], [0,1,0], @Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev, 0.degrees, 180.degrees, @Detail/2
@Elongation_Line12_2 = @Anchor.entities.add_line [(@Hole_Diameter/2+@Anchor_Thickness)-@Anc_Bev, @Anc_Depth, @Half_Susp_Length], [@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev, @Anc_Depth, @Half_Susp_Length-@Total_Elongation]
@Elongation_Line22_2 = @Anchor.entities.add_line [-(@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev), @Anc_Depth, @Half_Susp_Length], [-(@Hole_Diameter/2+@Anchor_Thickness-@Anc_Bev), @Anc_Depth, @Half_Susp_Length-@Total_Elongation]

#External edges 2nd side
@End_Hole2_2 = @Anchor.entities.add_circle([0, @Anc_Depth, @Half_Susp_Length], [0, 1, 0] , @Hole_Diameter/2, @Detail)
@Anchor_Arc_2 = @Anchor.entities.add_arc [0, @Anc_Depth, @Half_Susp_Length], [-1,0,0], [0,1,0], @Hole_Diameter/2+@Anchor_Thickness, 0.degrees, 180.degrees, @Detail/2
@Elongation_Line1_2 = @Anchor.entities.add_line [(@Hole_Diameter/2+@Anchor_Thickness), @Anc_Depth, @Half_Susp_Length], [@Hole_Diameter/2+@Anchor_Thickness, @Anc_Depth, @Half_Susp_Length-@Total_Elongation]
@Elongation_Line2_2 = @Anchor.entities.add_line [-(@Hole_Diameter/2+@Anchor_Thickness), @Anc_Depth, @Half_Susp_Length], [-(@Hole_Diameter/2+@Anchor_Thickness), @Anc_Depth, @Half_Susp_Length-@Total_Elongation]


Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@End_Hole2_2)
Sketchup.active_model.selection.add(@Anchor_Arc_2)
Sketchup.active_model.selection.add(@Elongation_Line1_2)
Sketchup.active_model.selection.add(@Elongation_Line2_2)

transform = Geom::Transformation.new([ 0, -@Anc_Bev, 0 ])
@Anchor.entities.transform_entities(transform, @Elongation_Line1_2)
@Anchor.entities.transform_entities(transform, @Elongation_Line2_2)
@Anchor.entities.transform_entities(transform, @End_Hole2_2)

@Anchor = @Anchor.move! [0, -@Anc_Depth/2, 0]
@End = @End.to_component

#2nd END
@End2 = @End.copy
transformation = Geom::Transformation.rotation([0,0,0], [0,1,0], 180.degrees)
@End2.transform!(transformation)
)

print 'Make Adjustable Suspension - Debug point 6/7 ok'

(# COMPRESSOR--------------------------------------------------------------------
#COMPRESSOR
@Compressor = @Structure.entities.add_group
@Comp_Bevel = @Compressor_Thickness/4

@Compressor_Circle1 = @Compressor.entities.add_circle([ 0, 0, (-@Half_Range+@Casing_Length-@Compression_Range)], [0, 0, 1] , @Casing_Out_Radius, @Detail)
@Compressor_Circle2  = @Compressor.entities.add_circle([ 0, 0, (-@Half_Range+@Casing_Length-@Compression_Range)], [0, 0, 1] , @Plates_Radius, @Detail)

@Compressor_Face1 = @Compressor.entities.add_face(@Compressor_Circle1)
@Compressor_Face2 = @Compressor.entities.add_face(@Compressor_Circle2)
@Compressor_Face1.erase!

if @Compressor_Face2.normal.z == -1
@Compressor_Face2.reverse!
end

#Apply face color
@Compressor_Face2.material = @Compressor_Color
@Compressor_Face2.back_material = @Compressor_Color

@Compressor_Face2.pushpull(-@Compressor_Thickness, true)


@Compressor_Circle3  = @Compressor.entities.add_circle([ 0, 0, (-@Half_Range+@Casing_Length-@Compression_Range)], [0, 0, 1] , @Plates_Radius-@Comp_Bevel, @Detail)
@Compressor_Circle4  = @Compressor.entities.add_circle([ 0, 0, (-@Half_Range+@Casing_Length-@Compression_Range-@Compressor_Thickness)], [0, 0, 1] , @Plates_Radius-@Comp_Bevel, @Detail)
@Compressor_Circle5  = @Compressor.entities.add_circle([ 0, 0, (-@Half_Range+@Casing_Length-@Compression_Range-@Compressor_Thickness)], [0, 0, 1] , @Plates_Radius, @Detail)


Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Compressor_Circle5)

transform = Geom::Transformation.new([ 0, 0, -@Comp_Bevel ])
@Compressor.entities.transform_entities(transform, @Compressor_Circle1)
@Compressor.entities.transform_entities(transform, @Compressor_Circle2)

transform = Geom::Transformation.new([ 0, 0, @Comp_Bevel ])
@Compressor.entities.transform_entities(transform, @Compressor_Circle5)
)

if @MakeRef == "No"
@Reference.erase!
end


print 'Make Adjustable Suspension - Debug point 7/7 ok'
#Reset Scale
scale_transform = Geom::Transformation.scaling([0,0,0], 1/@Scale)
@Suspension.transformation *= scale_transform
Sketchup.active_model.selection.clear
#--------------------------------------------------------------------------

 #Timer end
@end_time = Time.now
@Total_Time = @end_time - @start_time
UI.messagebox("Success. (" + @Total_Time.to_s + " seconds)")
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
#--------------------------------------------------------------------------
end

def self.bicycle_wheel_dialog
#Colors
@Hub_Color = [35,35,35]
@Spokes_Color = [60,60,60]
@Rim_Color = [85,85,85]
@Tyre_Color = [10,10,10]
@Scale = 1000.00
@Hub_Holes_Factor = 0.5 #Radial distance on hub plate

@Detail = 36
@Rim_Inner_Diameter = 647.00.mm
@Hub_Hole_Diameter = 20.00.mm
@N_Spokes = 20
@Spoke_Diameter = 2.50.mm
@Hub_Width = 100.00.mm
@Rim_Width = 24.00.mm
@Hub_Thickness = 5.00.mm
@Rim_Thickness = 2.00.mm
@Tyre_Thickness = 5.00.mm
@Rim_Height = 26.00.mm
@Tyre_Height = 50.00.mm
@Hub_Plate_Offset = 70.00
@Spoke_Hub_Plate_Ratio = 6
@Rim_Plate_Offset = 50.00
@MakeRef = "Yes"


@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["General detail: ", "Rim diameter: ", "Hub hole diameter: ", "Spokes: ", "Spoke diameter: ", "Hub width: ", "Rim width: ", "Hub thickness: ", "Rim thickness: ", "Tyre thickness: ", "Rim height: ", "Tyre height: ", "Hub plate offset (0-100%): ", "Spoke - Hub plate ratio: ", "Rim plate offset (0-100%): ", "Make references: "]
defaults = [@Detail, @Rim_Inner_Diameter, @Hub_Hole_Diameter, @N_Spokes, @Spoke_Diameter, @Hub_Width, @Rim_Width, @Hub_Thickness, @Rim_Thickness, @Tyre_Thickness, @Rim_Height, @Tyre_Height, @Hub_Plate_Offset, @Spoke_Hub_Plate_Ratio, @Rim_Plate_Offset, @MakeRef]
list = ["", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "Yes|No"]

val = UI.inputbox(prompts, defaults, list, "Make Bicycle Wheel")
if val
@Detail, @Rim_Inner_Diameter, @Hub_Hole_Diameter, @N_Spokes, @Spoke_Diameter, @Hub_Width, @Rim_Width, @Hub_Thickness, @Rim_Thickness, @Tyre_Thickness, @Rim_Height, @Tyre_Height, @Hub_Plate_Offset, @Spoke_Hub_Plate_Ratio, @Rim_Plate_Offset, @MakeRef = val;
else
 abort("Make Bicycle Wheel error: BWxV.000 - Operation canceled.\n")
end
#-------------------------------------------------------------------------------


#Log and verification
print 'Make Bicycle Wheel operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s + ' | ' + val[8].to_s + ' | ' + val[9].to_s + ' | ' + val[10].to_s + ' | ' + val[11].to_s + ' | ' + val[12].to_s + ' | ' + val[13].to_s + ' | ' + val[14].to_s + ' | ' + val[15].to_s


(#VARIABLES

@Bevel = @Hub_Thickness/2.0
@Bevel2 = @Hub_Thickness/4.0

@N_Spokes = (@N_Spokes.to_i).abs
if @N_Spokes%2 != 0 then @N_Spokes += 1; end
@Spoke_Angular_Offset = ( 360.000.degrees / (2.00 * @N_Spokes) )

@Hub_Plate_Height = @Spoke_Diameter * @Spoke_Hub_Plate_Ratio

@Spoke_Holes_Radius = @Hub_Hole_Diameter/2.0 + @Hub_Thickness + (@Hub_Plate_Height - @Bevel) *@Hub_Holes_Factor
#-------------------------------------------------------------------------------
)

print 'Make Bicycle Wheel - Debug point 1/7 ok'

(#VERIFICATIONS

#Detail < 6
if (@Detail < 8)
UI.messagebox("error: BWxV.001 - Min general detail is 8.")
warn("Make Parts error: BWxV.001")
@Verification = "NOT OK"
end

#Spokes > 1
if (@N_Spokes <= 1)
UI.messagebox("error: BWxV.002 - Min spokes is 2.")
warn("Make Parts error: BWxV.002")
@Verification = "NOT OK"
end

if (@Hub_Width < @Rim_Width)
UI.messagebox("error: BWxV.003 - Hub width must be larger than rim width.")
warn("Make Parts error: BWxV.003")
@Verification = "NOT OK"
end

if ( (@Tyre_Thickness *1.2) >= (@Rim_Width - 2*@Rim_Thickness)/2.0 )
UI.messagebox("error: BWxV.004 - Tyre or Rim thickness are too large; or rim width is too small.")
warn("Make Parts error: BWxV.004")
@Verification = "NOT OK"
end

if (@Rim_Height <= (@Rim_Width/2.0) )
UI.messagebox("error: BWxV.005 - Rim height is too low.")
warn("Make Parts error: BWxV.005")
@Verification = "NOT OK"
end

if (@Tyre_Height <= 0)
UI.messagebox("error: BWxV.006 - Tyre height must be higher than 0.")
warn("Make Parts error: BWxV.006")
@Verification = "NOT OK"
end

if (@Hub_Hole_Diameter <= 0)
UI.messagebox("error: BWxV.007 - Hub hole diameter must be higher than 0.")
warn("Make Parts error: BWxV.007")
@Verification = "NOT OK"
end

if (@Rim_Inner_Diameter <= (@Hub_Hole_Diameter + @Hub_Thickness*2 + @Hub_Plate_Height*2)*1.5 )
UI.messagebox("error: BWxV.008 - Rim diameter is too small.")
warn("Make Parts error: BWxV.008")
@Verification = "NOT OK"
end

if (@Hub_Plate_Offset < 0) or (@Hub_Plate_Offset > 100)
UI.messagebox("error: BWxV.009 - Hub plate offset is out of range (0-100%).")
warn("Make Parts error: BWxV.009")
@Verification = "NOT OK"
end

if (@Rim_Plate_Offset < 0) or (@Rim_Plate_Offset > 100)
UI.messagebox("error: BWxV.010 - Rim plate offset is out of range (0-100%).")
warn("Make Parts error: BWxV.010")
@Verification = "NOT OK"
end


if ( ((@N_Spokes/2.0) * (2*@Bevel2 + @Spoke_Diameter) * 1.2) >= (2 * Math::PI * @Spoke_Holes_Radius) )
UI.messagebox("error: BWxV.011 - Too much spokes.")
warn("Make Parts error: BWxV.011")
@Verification = "NOT OK"
end

if ( @Spoke_Hub_Plate_Ratio < 2 )
UI.messagebox("error: BWxV.012 - Min Spoke-Hub_plate ratio is 2.")
warn("Make Parts error: BWxV.012")
@Verification = "NOT OK"
end

if ( (2*@Bevel2 + @Spoke_Diameter) * 1.2 >= (@Hub_Plate_Height - @Bevel) )
UI.messagebox("error: BWxV.013 - Spoke - Hub plate ratio is too low.")
warn("Make Parts error: BWxV.013")
@Verification = "NOT OK"
end

if ( @Spoke_Diameter <= 0 )
UI.messagebox("error: BWxV.014 - Spoke diameter must be higher than 0.")
warn("Make Parts error: BWxV.014")
@Verification = "NOT OK"
end

if ( @Hub_Thickness >= @Hub_Width/4.0 )
UI.messagebox("error: BWxV.015 - Hub thickness may be too high.")
warn("Make Parts error: BWxV.015")
@Verification = "NOT OK"
end
)

end
# SET VARIABLES ON SCALE PT1 -------------------------------------------------------------------------------

@Tyre_Thickness = (@Tyre_Thickness * @Scale)
@Tyre_Height = (@Tyre_Height * @Scale)

@Hub_Hole_Diameter = (@Hub_Hole_Diameter * @Scale)
@Hub_Width = (@Hub_Width * @Scale)
@Hub_Thickness = (@Hub_Thickness * @Scale)

@Spoke_Diameter = (@Spoke_Diameter * @Scale)

@Rim_Inner_Diameter = (@Rim_Inner_Diameter * @Scale)
@Rim_Height = (@Rim_Height * @Scale)
@Rim_Width = (@Rim_Width * @Scale)
@Rim_Thickness = (@Rim_Thickness * @Scale)

# SET VARIABLES ON SCALE PT2 -------------------------------------------------------------------------------

@Bevel = @Hub_Thickness/2.0
@Bevel2 = @Hub_Thickness/4.0

@Hub_Plate_Height = @Spoke_Diameter * @Spoke_Hub_Plate_Ratio

@Spoke_Holes_Radius = @Hub_Hole_Diameter/2.0 + @Hub_Thickness + (@Hub_Plate_Height - @Bevel) *@Hub_Holes_Factor
#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------
print 'Make Bicycle Wheel - Debug point 2/7 ok'

@Continue = UI.messagebox("It can take a few minutes, please wait.", type = MB_OKCANCEL)
if (@Continue != IDOK) then abort("Make Bicycle Wheel error: BWxV.000 - Operation canceled.\n"); end

#Timer start
@start_time = Time.now

#Define camera view ------------------------------------------------------------
camera = Sketchup::Camera.new([0, -200, 0], [0, 0, 0], Z_AXIS)
view = Sketchup.active_model.active_view
view.camera = camera

#Speed boost by disable UI updates
model = Sketchup.active_model
entities = model.active_entities
model.start_operation('Make Parts', true)
#-------------------------------------------------------------------------------


#Make Bicycle Wheel
model = Sketchup.active_model
entities = model.active_entities
@Wheel = Sketchup.active_model.entities.add_group
@Hub = @Wheel.entities.add_group
@Hub_Holes = @Wheel.entities.add_group

#SOME HUB PARAMETERS ----------------------------------------------------------------
@Plate_Min = (@Spoke_Diameter + @Hub_Thickness)
@Plate_Max = (@Hub_Width/2.0) - @Bevel
@Plate_Position = ( ((@Plate_Max - @Plate_Min)/100.00) * @Hub_Plate_Offset) + @Plate_Min
#------------------------------------------------------------------------------------

#Rim Plate Position ----------------------------------------------------------------
@Top_Rim = (@Rim_Inner_Diameter/2.0) + @Rim_Height
@Rim_Plate_Min = @Top_Rim
@Rim_Plate_Max = (@Rim_Inner_Diameter/2.0) + (@Rim_Width/2.0) + @Rim_Thickness 
@Rim_Plate_Position = ( ((@Rim_Plate_Max - @Rim_Plate_Min)/100.00) * @Rim_Plate_Offset) + @Rim_Plate_Min
#------------------------------------------------------------------------------------



(#MAKING HUB SILHOUETE
@Point1 = [0, ((@Hub_Width/2.0)-(@Bevel)), (@Hub_Hole_Diameter/2.0)]
@Point2 = [0, -((@Hub_Width/2.0)-(@Bevel)), (@Hub_Hole_Diameter/2.0)]
@Base_Line = @Hub.entities.add_line @Point1, @Point2

@Arc1 = @Hub.entities.add_arc [0, @Point1.y, @Point1.z+@Bevel], [0,0,-1], [1,0,0], @Bevel, 0.degrees, 180.degrees, (@Detail/4.0)+2
@Arc2 = @Hub.entities.add_arc [0, @Point2.y, @Point2.z+@Bevel], [0,0,1], [1,0,0], @Bevel, 0.degrees, 180.degrees, (@Detail/4.0)+2

@Point3 = [0,@Point1.y, @Point1.z + @Hub_Thickness]
@Point4 = [0,@Point2.y, @Point2.z + @Hub_Thickness]

@Line2 = @Hub.entities.add_line @Point3, [0, @Plate_Position, @Point3.z]
@Line3 = @Hub.entities.add_line @Point4, [0, -@Plate_Position, @Point4.z]

#HUB PLATE 1
@Hub_Plate1_1 = @Hub.entities.add_line [0, @Plate_Position, @Point3.z], [0, @Plate_Position, (@Point3.z + @Hub_Plate_Height - @Bevel)]

@Hub_Plate1_2 = @Hub.entities.add_line [0, (@Plate_Position-@Hub_Thickness), @Point3.z], [0, (@Plate_Position-@Hub_Thickness), (@Point3.z + @Hub_Plate_Height - @Bevel)]

@Hub_Plate1_Arc = @Hub.entities.add_arc [0, (@Plate_Position-(@Hub_Thickness/2.0)), (@Point3.z + @Hub_Plate_Height - @Bevel)], [0,1,0], [1,0,0], @Bevel, 0.degrees, 180.degrees, (@Detail/4.0)+2


#HUB PLATE 2
@Hub_Plate2_1 = @Hub.entities.add_line [0, -@Plate_Position, @Point4.z], [0, -@Plate_Position, (@Point4.z + @Hub_Plate_Height - @Bevel)]

@Hub_Plate2_2 = @Hub.entities.add_line [0, -(@Plate_Position-@Hub_Thickness), @Point4.z], [0, -(@Plate_Position-@Hub_Thickness), (@Point4.z + @Hub_Plate_Height - @Bevel)]

@Hub_Plate2_Arc = @Hub.entities.add_arc [0, -(@Plate_Position-(@Hub_Thickness/2.0)), (@Point4.z + @Hub_Plate_Height - @Bevel)], [0,1,0], [1,0,0], @Bevel, 0.degrees, 180.degrees, (@Detail/4.0)+2


@Link_Line = @Hub.entities.add_line [0, (@Plate_Position-@Hub_Thickness), @Point3.z], [0, -(@Plate_Position-@Hub_Thickness), @Point4.z]


#Make hub silhouete
Sketchup.active_model.selection.clear
@Silhouete_Path = @Base_Line.all_connected
@Silhouete_Face = @Hub.entities.add_face(@Silhouete_Path)
if @Silhouete_Face.normal.x == -1 then @Silhouete_Face.reverse!; end
)

#Make 3d hub

#MAKE FOLLOWME PATH 1
@Rim_Circle1 = @Hub.entities.add_circle([0,0,0], [0, 1, 0] , @Rim_Inner_Diameter/2.0, @Detail)
@Rim_Path1 = @Rim_Circle1.to_a

@Silhouete_Face.followme(@Rim_Path1)
@Rim_Path1.each { |edge| edge.erase! }

(#Make hub holes
@Spoke_Holes_Radius = @Hub_Hole_Diameter/2.0 + @Hub_Thickness + (@Hub_Plate_Height - @Bevel) *@Hub_Holes_Factor
@Spoke_Holes_Offset = 360.00.degrees / (@N_Spokes/2.0)

for step in 0..(@N_Spokes/2.0) -1
@x = Math.cos(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius
@y = -@Hub_Width/2.0
@z = Math.sin(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius

@Circle1 = @Hub_Holes.entities.add_circle([@x, @y, @z], [0, 1, 0] , (@Spoke_Diameter/2.0), @Detail)
@F1 = @Hub_Holes.entities.add_face(@Circle1)
@F1.pushpull(@Hub_Width, false)
end
@Hub = @Hub_Holes.subtract(@Hub)
)

print 'Make Bicycle Wheel - Debug point 3/7 ok'

(#Make hub holes BEVEL
Sketchup.active_model.selection.clear
transform1 = Geom::Transformation.new([0,-@Bevel2,0])
transform2 = Geom::Transformation.new([0,@Bevel2,0])
# HUB HOLES -------------------------------------------------------------------------
for step in 0..(@N_Spokes/2) -1
@x = Math.cos(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius
@y = @Plate_Position
@z = Math.sin(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius
# 1 ------------------------------------------------------------------------------------

@Circle1 = @Hub.entities.add_circle([@x, @y, @z], [0, 1, 0] , (@Spoke_Diameter/2.0), @Detail)
@Circle2 = @Hub.entities.add_circle([@x, @y, @z], [0, 1, 0] , (@Spoke_Diameter/2.0)+@Bevel2, @Detail)

Sketchup.active_model.selection.add(@Circle1)
@Hub.entities.transform_entities(transform1, @Circle1)
Sketchup.active_model.selection.clear

# 2 ------------------------------------------------------------------------------------

@Circle1 = @Hub.entities.add_circle([@x, @y - @Hub_Thickness, @z], [0, 1, 0] , (@Spoke_Diameter/2.0), @Detail)
@Circle2 = @Hub.entities.add_circle([@x, @y - @Hub_Thickness, @z], [0, 1, 0] , (@Spoke_Diameter/2.0)+@Bevel2, @Detail)

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Circle1)
@Hub.entities.transform_entities(transform2, @Circle1)
Sketchup.active_model.selection.clear


@x = Math.cos(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius
@y = -@Plate_Position +@Hub_Thickness
@z = Math.sin(step * @Spoke_Holes_Offset) * @Spoke_Holes_Radius
# 3 ------------------------------------------------------------------------------------

@Circle1 = @Hub.entities.add_circle([@x, @y, @z], [0, 1, 0] , (@Spoke_Diameter/2.0), @Detail)
@Circle2 = @Hub.entities.add_circle([@x, @y, @z], [0, 1, 0] , (@Spoke_Diameter/2.0)+@Bevel2, @Detail)

Sketchup.active_model.selection.add(@Circle1)
@Hub.entities.transform_entities(transform1, @Circle1)
Sketchup.active_model.selection.clear

# 4 ------------------------------------------------------------------------------------

@Circle1 = @Hub.entities.add_circle([@x, @y - @Hub_Thickness, @z], [0, 1, 0] , (@Spoke_Diameter/2.0), @Detail)
@Circle2 = @Hub.entities.add_circle([@x, @y - @Hub_Thickness, @z], [0, 1, 0] , (@Spoke_Diameter/2.0)+@Bevel2, @Detail)

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Circle1)
@Hub.entities.transform_entities(transform2, @Circle1)
Sketchup.active_model.selection.clear
end
#------------------------------------------------------------------------------------
)

@Hub.material = @Hub_Color

(#Make Rim Silhouete ---------------------------------------------------------------
@Rim = @Wheel.entities.add_group
@Rim_Holes = @Wheel.entities.add_group

#MAKE FOLLOWME PATH 2 - (for outer and inner parts of the rim)
@Rim_Circle2 = @Rim.entities.add_circle([0,0,0], [0, 1, 0] , @Rim_Inner_Diameter/4.0, @Detail)
@Rim_Path2 = @Rim_Circle2.to_a

@Rim_Center_Height = (@Rim_Inner_Diameter/2.0) + (@Rim_Width/2.0)


#Make outer path silhouete
@Rim_Base_Arc = @Rim.entities.add_arc [0, 0, @Rim_Center_Height], [0,1,0], [1,0,0], (@Rim_Width/2.0), 180.degrees, 360.degrees, (@Detail/2.0)

@L1 = @Rim.entities.add_line [0,(@Rim_Width/2.0),@Rim_Center_Height], [0,(@Rim_Width/2.0), @Top_Rim]
@L2 = @Rim.entities.add_line [0,-(@Rim_Width/2.0),@Rim_Center_Height], [0,-(@Rim_Width/2.0), @Top_Rim]

@Line_Plate_1 = @Rim.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position], [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position]

@Line_H_1 = @Rim.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim], [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position]
@Line_H_2 = @Rim.entities.add_line [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim], [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position]

@Line_Top_1 = @Rim.entities.add_line [0, ((@Rim_Width/2.0)), @Top_Rim], [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim]
@Line_Top_2 = @Rim.entities.add_line [0, -((@Rim_Width/2.0)), @Top_Rim], [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim]

#Make outer face
@Rim_Outer_Path = @Line_Plate_1.all_connected
@Rim_Outer_Face = @Rim.entities.add_face(@Rim_Outer_Path)

#Extrude outer
if @Rim_Outer_Face.normal.x == 1 then @Rim_Outer_Face.reverse!; end
@Rim_Solid = @Rim_Outer_Face.followme(@Rim_Path2)

#------------------------------------------------------------------------------------

#Make inner path silhouete
@Rim_Sup_Arc = @Rim.entities.add_arc [0, 0, @Rim_Center_Height], [0,1,0], [1,0,0], (@Rim_Width/2.0)-@Rim_Thickness, 180.degrees, 360.degrees, (@Detail/2.0)

@Line_Plate_2 = @Rim.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), (@Rim_Plate_Position-@Rim_Thickness)], [0, -((@Rim_Width/2.0)-@Rim_Thickness), (@Rim_Plate_Position-@Rim_Thickness)]

@Line_H_3 = @Rim.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), (@Rim_Plate_Position-@Rim_Thickness)], [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Center_Height]
@Line_H_4 = @Rim.entities.add_line [0, -((@Rim_Width/2.0)-@Rim_Thickness), (@Rim_Plate_Position-@Rim_Thickness)], [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Center_Height]

#Make inner face
@Rim_Inner_Path = @Line_Plate_2.all_connected
@Rim_Inner_Face = @Rim.entities.add_face(@Rim_Inner_Path)

#Extrude inner
if @Rim_Inner_Face.normal.x == -1 then @Rim_Inner_Face.reverse!; end
@Rim_Inner_Hole = @Rim_Inner_Face.followme(@Rim_Path2)


#Delete rim extrusion path
@Rim_Path2.each { |edge| edge.erase! }
)
#------------------------------------------------------------------------------------

print 'Make Bicycle Wheel - Debug point 4/7 ok'

(#Make Rim holes
@Rim_Holes_Offset = 360.00.degrees / @N_Spokes

for step in 0..(@N_Spokes-1)
@y = 0
@x = Math.cos(step * @Rim_Holes_Offset + @Spoke_Angular_Offset) * ((@Rim_Inner_Diameter/2.0)-(@Rim_Thickness*8))
@z = Math.sin(step * @Rim_Holes_Offset + @Spoke_Angular_Offset) * ((@Rim_Inner_Diameter/2.0)-(@Rim_Thickness*8))

@Circle2 = @Rim_Holes.entities.add_circle([@x, @y, @z], [@x, @y, @z] , (@Spoke_Diameter/2.0), @Detail)
@F2 = @Rim_Holes.entities.add_face(@Circle2)
@F2.pushpull(@Rim_Thickness*12, false)
end

@Rim = @Rim_Holes.subtract(@Rim)
)
@Rim.material = @Rim_Color


print 'Make Bicycle Wheel - Debug point 5/7 ok'

(#Make Tyre Silhouete
@Tyre = @Wheel.entities.add_group

@Top_Point = [0, 0, (@Rim_Inner_Diameter/2.0) + @Rim_Height + @Tyre_Height]
@Point_A = [0, ((@Rim_Width/2.0) - @Rim_Thickness), (@Rim_Inner_Diameter/2.0) + @Rim_Height]
@Point_B = [0, -((@Rim_Width/2.0) - @Rim_Thickness), (@Rim_Inner_Diameter/2.0) + @Rim_Height]

@Vector_1 = @Top_Point.vector_to( @Point_A )
@Vector_2 = @Top_Point.vector_to( @Point_B )

#Angle between 2 vectors ------------------------------------------------------------
@Angle1 = (@Vector_1.angle_between @Vector_2)
@Half_Angle = @Angle1/2.00

@CA = @Vector_1.length/2.00

@Hypotenuze = @CA / Math.cos(@Half_Angle)

@Center_Point = [0, 0, (@Rim_Inner_Diameter/2.0) + @Rim_Height + @Tyre_Height - @Hypotenuze]
#------------------------------------------------------------------------------------


@Vector_3 = @Center_Point.vector_to( @Point_A )
@Vector_4 = @Center_Point.vector_to( @Point_B )
@Angle2 = (@Vector_3.angle_between @Vector_4).radians

if (@Center_Point[2] >= @Point_A[2])
 @Tyre_Angle1 = 360.00 - @Angle2
else
 @Tyre_Angle1 = @Angle2
end

@Tyre_Outer_Arc = @Tyre.entities.add_arc @Center_Point, @Vector_3, [1,0,0], @Hypotenuze, 0, @Tyre_Angle1.degrees, @Detail
#------------------------------------------------------------------------------------

@Point_C = [0, ((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Top_Rim]
@Point_D = [0, -((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Top_Rim]
@Vector_5 = @Center_Point.vector_to( @Point_C )
@Vector_6 = @Center_Point.vector_to( @Point_D )
@Angle3 = (@Vector_5.angle_between @Vector_6).radians

if (@Center_Point[2] >= @Point_A[2])
 @Tyre_Angle2 = 360.00 - @Angle3
else
 @Tyre_Angle2 = @Angle3
end

@In_Rad = @Vector_5.length

@Tyre_Inner_Arc = @Tyre.entities.add_arc @Center_Point, @Vector_5, [1,0,0], (@Hypotenuze-@Tyre_Thickness), 0, @Tyre_Angle2.degrees, @Detail
#------------------------------------------------------------------------------------


@Tyre_Base_Line_1 = @Tyre.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position], [0, ((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Rim_Plate_Position]
@Tyre_Base_Line_2 = @Tyre.entities.add_line [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position], [0, -((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Rim_Plate_Position]

@Tyre_Line_1 = @Tyre.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim], [0, ((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position]
@Tyre_Line_2 = @Tyre.entities.add_line [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Top_Rim], [0, -((@Rim_Width/2.0)-@Rim_Thickness), @Rim_Plate_Position]


@Anglar_Reduction = (180.00 - @Tyre_Angle2) / 2.0

@y = @Center_Point.y + ( Math.cos(@Anglar_Reduction.degrees) * (@Hypotenuze-@Tyre_Thickness) )
@z = @Center_Point.z + ( Math.sin(@Anglar_Reduction.degrees) * (@Hypotenuze-@Tyre_Thickness) )
@Arc_Point_A = [ 0, @y, @z]
@Arc_Point_B = [ 0, -@y, @z]

@Tyre_Line_3 = @Tyre.entities.add_line [0, ((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Rim_Plate_Position], @Arc_Point_A
@Tyre_Line_4 = @Tyre.entities.add_line [0, -((@Rim_Width/2.0)-@Rim_Thickness-@Tyre_Thickness), @Rim_Plate_Position], @Arc_Point_B
#------------------------------------------------------------------------------------
)


#Make tyre 3d
@Tyre_Silhouete_Path = @Tyre_Line_1.all_connected
@Tyre_Silhouete_Face = @Tyre.entities.add_face(@Tyre_Silhouete_Path)

#MAKE FOLLOWME PATH 3
@Rim_Circle3 = @Tyre.entities.add_circle([0,0,0], [0, 1, 0] , @Rim_Inner_Diameter/2.0, @Detail)
@Rim_Path3 = @Rim_Circle3.to_a


if !(@Tyre_Silhouete_Face.nil?)
 if @Tyre_Silhouete_Face.normal.x == -1 then @Tyre_Silhouete_Face.reverse!; end
 @Tyre_Silhouete_Face.followme(@Rim_Path3)
 @Tyre.material = @Tyre_Color
end

@Rim_Path3.each { |edge| edge.erase! }
#------------------------------------------------------------------------------------

print 'Make Bicycle Wheel - Debug point 6/7 ok'

(#Make Spoke
@Spoke_Reference = @Wheel.entities.add_group
@Spokes = @Wheel.entities.add_group
@Spoke_Pair = @Spokes.entities.add_group
@Spoke = @Spoke_Pair.entities.add_group

#Link line
@Spoke_Curve_Int_Point = [@Spoke_Holes_Radius, (@Plate_Position-@Hub_Thickness),0]
@Spoke_Curve_Ext_Point = [@Spoke_Holes_Radius, (@Plate_Position),0]
@Spoke_Link_Line = @Spoke_Reference.entities.add_line @Spoke_Curve_Ext_Point, @Spoke_Curve_Int_Point

#Arcs
@Arc_Center_1 = [@Spoke_Holes_Radius+(@Spoke_Diameter/2.0), (@Plate_Position-@Hub_Thickness),0]
@Arc_Center_2 = [@Spoke_Holes_Radius+(@Spoke_Diameter/2.0), (@Plate_Position),0]

@Spoke_Arc_1 = @Spoke_Reference.entities.add_arc @Arc_Center_1, [1,0,0], [0,0,1], (@Spoke_Diameter/2.0), 180.degrees, 270.degrees, @Detail/4.0
@Spoke_Arc_2 = @Spoke_Reference.entities.add_arc @Arc_Center_2, [1,0,0], [0,0,1], (@Spoke_Diameter/2.0), 90.degrees, 180.degrees, @Detail/4.0

#End line 1
@Spoke_C1 = [@Spoke_Holes_Radius+(@Spoke_Diameter/2.0), (@Plate_Position+(@Spoke_Diameter/2.0)), 0]
@Spoke_End_Point1 = [@Spoke_C1[0] + (@Hub_Plate_Height/2.0), @Spoke_C1[1], 0]
@End_Line_1 = @Spoke_Reference.entities.add_line @Spoke_C1, @Spoke_End_Point1


#2nd end projection ------------------------------------------------------------------
@Spoke_C2 = [@Spoke_Holes_Radius+(@Spoke_Diameter/2.0), (@Plate_Position-@Hub_Thickness-(@Spoke_Diameter/2.0)), 0]

@Spoke_C2_proj = [@Spoke_Holes_Radius+(@Spoke_Diameter/2.0), 0, 0]

@x = Math.cos(@Spoke_Angular_Offset) * (@Rim_Inner_Diameter/2.0 + @Rim_Thickness)
@y = 0
@z = Math.sin(@Spoke_Angular_Offset) * (@Rim_Inner_Diameter/2.0 + @Rim_Thickness)
@End_Projection = [@x, @y, @z]

@Spoke_Vector1 = @Spoke_C2_proj.vector_to( @End_Projection )

@Spoke_End_Point2 = [@Spoke_C2[0] + @Spoke_C2_proj.distance(@End_Projection), 0, 0]
@Spoke_Vector2 = @Spoke_C2_proj.vector_to( @Spoke_End_Point2 )
@End_Line_2 = @Spoke_Reference.entities.add_line @Spoke_C2, @Spoke_End_Point2
#------------------------------------------------------------------------------------
)

#Make Spoke 3d
@Spoke_Path = @Spoke_Reference.entities[0].all_connected
@Spoke_Silhouete = @Spoke.entities.add_circle(@Spoke_End_Point1, @Spoke_End_Point1.vector_to(@Spoke_C1), (@Spoke_Diameter/2.0), @Detail)

@Spoke_Face = @Spoke.entities.add_face(@Spoke_Silhouete)

if @Spoke_Face.normal.x == -1 then @Spoke_Face.reverse!; end
@Spoke_Face.followme(@Spoke_Path)

(#Make spoke bevel
@Spoke_Bevel = @Spoke_Diameter/4.0

Sketchup.active_model.selection.clear

@Spoke_Bevel_Path1 = @Spoke.entities.add_circle(@Spoke_End_Point1, @Spoke_End_Point1.vector_to(@Spoke_C1), (@Spoke_Diameter/2.0), @Detail)
@Spoke_Bevel_Path2 = @Spoke.entities.add_circle(@Spoke_End_Point1, @Spoke_End_Point1.vector_to(@Spoke_C1), (@Spoke_Diameter/4.0), @Detail)

Sketchup.active_model.selection.add(@Spoke_Bevel_Path1)
transform = Geom::Transformation.new([-@Spoke_Bevel, 0, 0])
@Spoke.entities.transform_entities(transform, @Spoke_Bevel_Path1)

Sketchup.active_model.selection.clear
#------------------------------------------------------------------------------------
)

@Spoke.material = @Spokes_Color

#Angular Offset Rotation
@Spoke_Rot_Point = [@Spoke_Holes_Radius, (@Plate_Position-(@Hub_Thickness/2.0) ),0]
@Offset_Angle = @Spoke_Vector2.angle_between(@Spoke_Vector1)
@Spoke.entities.transform_entities(Geom::Transformation.rotation(@Spoke_Rot_Point, [0,-1,0], @Offset_Angle), @Spoke)
@Spoke = @Spoke.to_component

(#Copies
@Copy = @Spoke.copy
@Copy = entities.transform_entities(Geom::Transformation.rotation([0,0,0], [1,0,0], 180.degrees), @Copy)
@Spoke_Pair.material = @Spokes_Color

for step in 1..(@N_Spokes/2) -1
@Spokes_Angle = (step * @Spoke_Holes_Offset)
@Copy = @Spoke_Pair.copy
@Copy = entities.transform_entities(Geom::Transformation.rotation([0,0,0], [0,1,0], @Spokes_Angle), @Copy)
end
@Spokes.material = @Spokes_Color
)


@Spoke_Reference.erase!


#Make references
if (@MakeRef == "Yes")
@References = @Wheel.entities.add_group
@Origin_Point = @References.entities.add_cpoint [0, 0, 0]
@Axis_Line = @References.entities.add_line [0, @Hub_Width, 0], [0, -@Hub_Width, 0]
end

#Reset scale
scale_transform = Geom::Transformation.scaling([0,0,0], 1.0/@Scale)
@Wheel.transformation *= scale_transform

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Wheel)

#------------------------------------------------------------------------------------
print 'Make Bicycle Wheel - Debug point 7/7 ok'
@end_time = Time.now
@Total_Time = @end_time - @start_time
UI.messagebox("Success. (" + @Total_Time.to_s + " seconds)")
print "Success. (" + @Total_Time.to_s + " seconds) \n \n"
#------------------------------------------------------------------------------------
end

def self.railway_along_dialog
# SELECTION FILTER AND CHECK --------------------------------------------------------

model = Sketchup.active_model
entities = model.active_entities

@Raw_Selection = model.selection.to_a #Selection to array
@Raw_Edges = @Raw_Selection.grep(Sketchup::Edge) #Filter only edges

if (@Raw_Edges.length == 0)
UI.messagebox("No edges selected.")
abort("Make RailWay Along error: RAxV.000 - Operation aborted due to missing selection.")
end

#------------------------------------------------------------------------------------
@Reverse = "No"
@Make_Ref = "No"
@Timber_Bevel = "Yes"
@Bevel_Factor = 15.0
@Transition = "Linear"

@Rails_distance = 1435.00.mm #(GAUGE)
@Up_Offset = 50.00.mm

@Timber_Lenght = 2250.00.mm
@Timber_Width = 200.00.mm
@Timber_Thickness = 100.00.mm
@Timber_Spacing = 800.00.mm

@Rail_Color = [202,204,204]
@Timber_Color = [104,85,60]

#Rail profile variables
@HT = 5.75
@BW = 5.75
@HW = 2.75
@W = 0.5625
@HD = 1.703125
@FD = 3.078125
@BD = 0.96875
#------------------------------------------------------------------------------------
@Verification = "NOT OK"
until @Verification == "OK"
#-------------------------------------------------------------------------------
@Verification = "OK"
prompts = ["Rails gauge: ", "Timber lenght: ", "Timber width: ", "Timber thickness: ", "Timber spacing: ", "Up offset: ", "Reverse? ", "Timber bevel? ","Make Reference? ", "Transition mode: "]
defaults = [@Rails_distance, @Timber_Lenght, @Timber_Width, @Timber_Thickness, @Timber_Spacing, @Up_Offset, @Reverse, @Timber_Bevel, @Make_Ref, @Transition]
list = ["", "", "", "", "", "", "Yes|No", "Yes|No", "Yes|No", "Linear|Gradient"]

val = UI.inputbox(prompts, defaults, list, "Make Railway Along")
if val
@Rails_distance, @Timber_Lenght, @Timber_Width, @Timber_Thickness, @Timber_Spacing, @Up_Offset, @Reverse, @Timber_Bevel, @Make_Ref, @Transition = val;
else
 abort("Make Railway Along error: RAxV.001 - Operation canceled.\n")
end
#-------------------------------------------------------------------------------


#Log and verification
print 'Make RailWay Along operation:'
print 'Input log: ' + val[0].to_s + ' | ' + val[1].to_s + ' | ' + val[2].to_s + ' | ' + val[3].to_s + ' | ' + val[4].to_s + ' | ' + val[5].to_s + ' | ' + val[6].to_s + ' | ' + val[7].to_s + ' | ' + val[8].to_s
#------------------------------------------------------------------------------------
print 'Make RailWay Along - Debug point 1/10 ok' #DEBUG POINT <<< ################################## <<<


(#VERIFICATIONS

#@Rails_distance > @BW
if (@Rails_distance < @BW)
UI.messagebox("error: RAxV.002 - Rails gauge must be higher than " + @BW.to_s)
warn("Make Parts error: RAxV.002")
@Verification = "NOT OK"
end

if (@Timber_Lenght < (@Rails_distance + @BW) )
UI.messagebox("error: RAxV.003 - Timber lenght must be higher than " + (@Rails_distance + @BW).to_s)
warn("Make Parts error: RAxV.003")
@Verification = "NOT OK"
end

if (@Timber_Width > @Timber_Spacing )
UI.messagebox("error: RAxV.004 - Timber width must be smaller than timber spacing.")
warn("Make Parts error: RAxV.004")
@Verification = "NOT OK"
end

if (@Timber_Spacing <= 0 )
UI.messagebox("error: RAxV.005 - Timber spacing must be higher than 0.")
warn("Make Parts error: RAxV.005")
@Verification = "NOT OK"
end
)

end

#-------------------------------------------------------------------------------
print 'Make RailWay Along - Debug point 2/10 ok' #DEBUG POINT <<< ################################## <<<

#Handlers
model = Sketchup.active_model
entities = model.active_entities
#-------------------------------------------------------------------------------

Sketchup.active_model.selection.clear
Sketchup.active_model.selection.add(@Raw_Edges)

#Remove possible duplicates
@Raw_Edges.uniq!
@Edges_Ammount = @Raw_Edges.length

#Register all vertex for each edge
@Raw_Vertex = []
@Path_Length = 0
#------------------------------------------------------------------------------------
 @Raw_Edges.each do |edge|
  @Raw_Vertex << edge.start
  @Raw_Vertex << edge.end
  @Path_Length += edge.length
 end
 #------------------------------------------------------------------------------------

#OPERATION CONFIRMATION
@Sket_Version = Sketchup.version.to_i
#Estimate contruction speed based on my tests (Personal record: 14m/s on SK17 and 55m/s on SK18)
if (@Sket_Version > 17) then @Construction_Speed = 20.0; else @Construction_Speed = 0.5; end

@Total_in_seconds = ((@Path_Length / 1.00.m) / @Construction_Speed)
@Total_minutes_only = (@Total_in_seconds / 60.0).to_i
@Rest_in_seconds = (((@Total_in_seconds / 60.0) - @Total_minutes_only.to_f) * 60.0).round(2)
@Estimated_time_text = @Total_minutes_only.to_s + "min " + @Rest_in_seconds.to_s + "s"

@Text1 = "It can take a few minutes, please wait."
@Text2 = "Estimated time (at " + @Construction_Speed.to_s + "m/s): " + @Estimated_time_text

@Continue = UI.messagebox(@Text1 + "\n" + @Text2, type = MB_OKCANCEL)
if (@Continue != IDOK) then abort("Make Railway Along error: RAxV.001 - Operation canceled.\n"); end
#------------------------------------------------------------------------------------

#Timer start
@start_time = Time.now


#Get how many times each vertex is used
@Vertex_Counter = @Raw_Vertex.uniq.collect { |a| @Raw_Vertex.count(a) }

(# VERIFICATION (ends and derivations) ------------------------------------------------
if (@Vertex_Counter.count(1) > 2)
abort ("Error. More than 2 ends.")
end

if ((@Vertex_Counter.count(1) + @Vertex_Counter.count(2)) < @Vertex_Counter.length)
abort ("Error. There are derivations.")
end

@MODE = ""

if (@Vertex_Counter.count(1) == 2)
print ("Path type: open path.")
@MODE = "OPEN"
end

if (@Vertex_Counter.count(2) == @Vertex_Counter.length)
print ("Path type: closed loop.")
@MODE = "LOOP"
end
#------------------------------------------------------------------------------------
)


#GET FIRST VERTEX TO START SERIALIZE PROCESS
if (@MODE == "OPEN")
 @Start_Vertex = @Raw_Vertex.uniq[@Vertex_Counter.index(1)]
 @End_Edge_Position = (@Raw_Vertex.index(@Start_Vertex) / 2)
 @Initial_Edge = @Raw_Edges[@End_Edge_Position]
else
 @End_Edge_Position = 0
 @Initial_Edge = @Raw_Edges[@End_Edge_Position]
 @Start_Vertex = @Initial_Edge.start
end


print 'Make RailWay Along - Debug point 3/10 ok' #DEBUG POINT <<< ################################## <<<

#MAKE SERIAL POINTS ARRAY (SERIALIZE PROCESS)
@Current_Vertex = @Start_Vertex
@Serial_Points = []
@Serial_Points << @Current_Vertex.position

@Final_edge = (@Raw_Edges.length - 1)
@Last_Edge = ""
@Counter = 0
@State = ""
#------------------------------------------------------------------------------------#
until ((@Counter > (@Raw_Vertex.uniq.length-1)) or (@State == "End of path"))

@Current_Edge = 0
@State = "Not found"
 until ((@Current_Edge > @Final_edge) or (@State == "Found"))

  if ((@Current_Edge != @Last_Edge) and (@Raw_Edges[@Current_Edge].used_by? @Current_Vertex))
  @State = "Found"

  @Current_Vertex = @Raw_Edges[@Current_Edge].other_vertex @Current_Vertex

   @Serial_Points << @Current_Vertex.position

  @Last_Edge = @Current_Edge
  end

 @Current_Edge += 1
 end

 if (@State == "Not found")
 @State = "End of path"
 end

@Counter += 1
end



#REMOVE POINTS ON STRAIGHT LINES
@Points_To_Remove = []
@Final_Point = (@Serial_Points.length-1)
#------------------------------------------------------------------------------------
@Serial_Points.each do |position|

#Point ID in array
@ID = @Serial_Points.index(position)

#IF NOT FIRST OR LAST POINT
 if ((@ID != 0) and (@ID != @Final_Point))

#POINTS (3)
@Current_Point = position
@Ahead_Point = @Serial_Points[@ID+1]
@Back_Point = @Serial_Points[@ID-1]

#VECTORS (2)
@Last_Vector = (@Back_Point.vector_to( @Current_Point )).normalize
@Next_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize

 #Register point to delete
 if (@Last_Vector == @Next_Vector)
  @Points_To_Remove << @Current_Point
 end
end

end
#------------------------------------------------------------------------------------
@Serial_Points = @Serial_Points - @Points_To_Remove #Remove the unecessary points


#REVERSE PATH?
if (@Reverse == "yes") then @Serial_Points.reverse!; end

print 'Make RailWay Along - Debug point 4/10 ok' #DEBUG POINT <<< ################################## <<<

#MAKE POINTS AXES
@Points_Axes = [] # <- [Z, SIDE1, SIDE2, AVER, AHEAD]
@Curves_State = ""
@Final_Point = (@Serial_Points.length-1)

#------------------------------------------------------------------------------------
@Serial_Points.each do |position|
#Point ID in array
@ID = @Serial_Points.index(position)

#POINTS (3)
@Current_Point = @Serial_Points[@ID]
@Ahead_Point = @Serial_Points[@ID+1]
@Back_Point = @Serial_Points[@ID-1]


#IF FIRST POINT - # - # - # - # - # - # - # - # - # - # - # - # - #
 if @ID == (0)
@Up_Vector_Basis = Z_AXIS
@Average_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Ahead_Vector = @Average_Vector

if (@Average_Vector[2].abs == 1)
UI.messagebox("Aborted, first edge direction cannot be on Z_AXIS.")
abort("Aborted, 1st edge on Z_AXIS.")
end

@Side_Vector1 = (@Average_Vector.cross(@Up_Vector_Basis)).normalize
@Side_Vector2 = @Side_Vector1.reverse
 
@Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
if (@Up_Vector.z < 0) then @Up_Vector.reverse!; end

#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (first point)
else

#IF FINAL POINT - # - # - # - # - # - # - # - # - # - # - # - # - #
 if @ID == (@Final_Point)
@Average_Vector = (@Back_Point.vector_to( @Current_Point )).normalize
@Ahead_Vector = @Average_Vector

@Up_Vector_Prediction = @Up_Vector_H
if ((@Up_Vector_Prediction.angle_between @Up_Vector_H).radians > 90) then @Up_Vector_Prediction.reverse!; end

@Side_Vector1 = (@Average_Vector.cross(@Up_Vector_Prediction)).normalize
if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
@Side_Vector2 = @Side_Vector1.reverse

@Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 
#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (final point)
 else # - # - # - # - # - # - # - # - # - # - # - # - # - #

#AVERAGE VECTOR
@Serial_Vector1 = (@Back_Point.vector_to( @Current_Point )).normalize
@Serial_Vector2 = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Average_Vector = (@Serial_Vector1 + @Serial_Vector2).normalize

#PROJECTION VECTORS (2)
@Ahead_Vector = (@Current_Point.vector_to( @Ahead_Point )).normalize
@Back_Vector = (@Current_Point.vector_to( @Back_Point )).normalize

if ((@Ahead_Vector.angle_between @Back_Vector).radians < 120)
@Curves_State = "BAD"
@pt = entities.add_cpoint @Current_Point
end

#TOP PROJECTIONS (2)
@Ahead_Top_Project_Vector = Geom::Vector3d.new(@Ahead_Vector[0], @Ahead_Vector[1], 0).normalize
@Back_Top_Project_Vector = Geom::Vector3d.new(@Back_Vector[0], @Back_Vector[1], 0).normalize
@Curve_Angle = (@Ahead_Top_Project_Vector.angle_between @Back_Top_Project_Vector).radians

#Define up and side vectors
if (@Curve_Angle == 180 or @Curve_Angle == 0 or @Ahead_Top_Project_Vector == [0,0,0] or @Back_Top_Project_Vector == [0,0,0])
 @Up_Vector = (@Back_Vector + @Ahead_Vector).normalize

     #Attempt to solve precision issue with rlly smooth curves
      if (@Up_Vector == [0,0,0]) then @Up_Vector = @Up_Vector_H; end

 if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 @Side_Vector1 = (@Average_Vector.cross(@Up_Vector)).normalize
 if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
 @Side_Vector2 = @Side_Vector1.reverse

else

 @Side_Vector1 = (@Back_Top_Project_Vector + @Ahead_Top_Project_Vector).normalize
 
      if (@Side_Vector1 == [0,0,0]) #Attempt to solve precision issue with rlly smooth curves
       @Side_project1 = @Ahead_Top_Project_Vector.cross(@Up_Vector_H)
       @Side_project2 = Back_Top_Project_Vector.cross(@Up_Vector_H)

       @Current_Lateral = [@Current_Point, @Side_project1]
       @Back_Lateral = [@Back_Point, @Side_project2]

       @Intersection = Geom.intersect_line_line(@Current_Lateral, @Back_Lateral)
       @Side_Vector1 = @Current_Point.vector_to(@Intersection).normalize
      end
 
 if ((@Side_Vector1.angle_between @Side_Vector1_H).radians > 90) then @Side_Vector1.reverse!; end
 @Up_Vector = (@Average_Vector.cross(@Side_Vector1)).normalize
 if ((@Up_Vector.angle_between @Up_Vector_H).radians > 90) then @Up_Vector.reverse!; end
 @Side_Vector2 = @Side_Vector1.reverse
end

#Register axes for current point
@Points_Axes << [@Up_Vector, @Side_Vector1, @Side_Vector2, @Average_Vector, @Ahead_Vector] # <<<<<<<<<< REGISTER (middle points)
 end
end

#Past vectors historic
@Up_Vector_H = @Up_Vector
@Side_Vector1_H = @Side_Vector1
end
#------------------------------------------------------------------------------------
if (@Curves_State == "BAD")
@Continue = UI.messagebox("There are hard curves (c < 120°). Consider smoothing the path.\nContinue operation?", type = MB_YESNO)
if (@Continue == IDNO) then abort ("Operation aborted due to hard curves (c < 120°)."); end
end

print 'Make RailWay Along - Debug point 5/10 ok' #DEBUG POINT <<< ################################## <<<

# Outer group -----------------------------------------------------------------------
@RailWay = Sketchup.active_model.entities.add_group
#------------------------------------------------------------------------------------


#MAKE SIDES PATHS -------------------------------------
Sketchup.active_model.start_operation "Railroad along op.1"

(#Initial definitions for side paths
@Rails_Offset = (@Rails_distance + @HW)/2.00
@Half_Timber = (@Timber_Lenght / 2.00)

#Intersections status
@Status_1 = ""
@Status_2 = ""
@Status_Up = ""

#Sides Groups
@Side1_ref = @RailWay.entities.add_group
@Side2_ref = @RailWay.entities.add_group

#Initial historic self reference
@Side_Point1_H = @Serial_Points[0].offset(@Points_Axes[0][1], @Rails_Offset)
@Side_Point2_H = @Serial_Points[0].offset(@Points_Axes[0][2], @Rails_Offset)
@Up_Point_H = @Serial_Points[0].offset(@Points_Axes[0][0], @Rails_Offset)
)

@Serial_Points.each do |position|
#Point ID in array
@ID = @Serial_Points.index(position)
@Current_Point = @Serial_Points[@ID]

#Get axes[Z, SIDE1, SIDE2, AHEAD]
@Up_Vector = @Points_Axes[@ID][0]
@Side1_Vector = @Points_Axes[@ID][1]
@Side2_Vector = @Points_Axes[@ID][2]
@Ahead_Vector = @Points_Axes[@ID][3]

#Get current points
@Side_Point1 = @Current_Point.offset(@Side1_Vector, @Rails_Offset)
@Side_Point2 = @Current_Point.offset(@Side2_Vector, @Rails_Offset)
@Up_Point = @Current_Point.offset(@Up_Vector, @Rails_Offset)


#Make lines
@Side_Line_1 = @Side1_ref.entities.add_line(@Side_Point1_H, @Side_Point1)
@Side_Line_2 = @Side2_ref.entities.add_line(@Side_Point2_H, @Side_Point2)
#@Up_Line = entities.add_line(@Up_Point_H, @Up_Point) # <<<<< TEMPORARY DEBUG


#Check intersections -----------------------
if (@ID != 0)
@Side1_line1 = [@Current_Point, (@Current_Point.vector_to @Side_Point1)]
@Side1_line2 = [@Last_Point, (@Last_Point.vector_to @Side_Point1_H)]
@INT1 = Geom.intersect_line_line(@Side1_line1, @Side1_line2)
if ((@INT1 != nil) and (@Current_Point.distance(@INT1) < @Half_Timber)) then @Status_1 = "BAD"; @pt = entities.add_cpoint @Current_Point; end

@Side2_line1 = [@Current_Point, (@Current_Point.vector_to @Side_Point2)]
@Side2_line2 = [@Last_Point, (@Last_Point.vector_to @Side_Point2_H)]
@INT2 = Geom.intersect_line_line(@Side2_line1, @Side2_line2)
if ((@INT2 != nil) and (@Current_Point.distance(@INT2) < @Half_Timber)) then @Status_2 = "BAD"; @pt = entities.add_cpoint @Current_Point; end

@Up_line1 = [@Current_Point, (@Current_Point.vector_to @Up_Point)]
@Up_line2 = [@Last_Point, (@Last_Point.vector_to @Up_Point_H)]
@INT3 = Geom.intersect_line_line(@Up_line1, @Up_line2)
if ((@INT3 != nil) and (@Current_Point.distance(@INT3) < @Half_Timber)) then @Status_Up = "BAD"; @pt = entities.add_cpoint @Current_Point; end
end
#-----------------------------------------------

#Make points historic
@Side_Point1_H = @Side_Point1
@Side_Point2_H = @Side_Point2
@Up_Point_H = @Up_Point
@Last_Point = @Current_Point
end

Sketchup.active_model.commit_operation
#------------------------------------------------------------------------------------

#Intersection advise ----------------------------------------------------------------
if (@Status_1 == "BAD") or (@Status_2 == "BAD") or (@Status_Up == "BAD")
 UI.messagebox("Operation aborted!\nThere are intersections.\nRails distance is too large or curves radius are too low.")
 Sketchup.undo
 abort ("Operation aborted due to intersections.")
end

if (@Make_Ref == "No")
@Side1_ref.erase!
@Side2_ref.erase!
end
#------------------------------------------------------------------------------------

print 'Make RailWay Along - Debug point 6/10 ok' #DEBUG POINT <<< ################################## <<<


Sketchup.active_model.start_operation "Railroad along op.2"
#MAKE RAIL PROFILE ------------------------------------------------------ <<<<<<<<<<<
@Rails1 = @RailWay.entities.add_group
@Rails2 = @RailWay.entities.add_group
@Rail_Profile = @RailWay.entities.add_group

@Points = []
@Points[0] = [(@BW/2.0), 0, 0]
@Points[1] = [(@BW/2.0), 0, @BD - (((@BW/2.0)-(@W/2.0))/4.0)]
@Points[2] = [(@W/2.0), 0, @BD]
@Points[3] = [(@W/2.0), 0, @BD + @FD]
@Points[4] = [(@HW/2.0), 0, @BD + @FD + (((@HW-@W)/2.0)/4.0)]
@Points[5] = [(@HW/2.0), 0, @HT-(@HD/4.0)]
@Points[6] = [((@HW/2.0)-(@HW/8.0)), 0, @HT]

@Points[14] = [(@BW/2.0), 0, 0]
@Points[13] = [-(@BW/2.0), 0, 0]
@Points[12] = [-(@BW/2.0), 0, @BD - (((@BW/2.0)-(@W/2.0))/4.0)]
@Points[11] = [-(@W/2.0), 0, @BD]
@Points[10] = [-(@W/2.0), 0, @BD + @FD]
@Points[9] = [-(@HW/2.0), 0, @BD + @FD + (((@HW-@W)/2.0)/4.0)]
@Points[8] = [-(@HW/2.0), 0, @HT-(@HD/4.0)]
@Points[7] = [-((@HW/2.0)-(@HW/8.0)), 0, @HT]

@Silhouete = @Rail_Profile.entities.add_line @Points
@Profile_def = @Rail_Profile.definition
#------------------------------------------------------------------------------------

@Last_Point = (@Serial_Points.length-1)
@Last_ID = 0

#Rails side 1 --------------------------------------
@Serial_Points.each do |position|
@ID = @Serial_Points.index(position)

#Add new rail profile
@Rail_Copy = @Rails1.entities.add_instance @Profile_def, Sketchup.active_model.axes.transformation

#Get rails profile points
@Current_Points = []
@Rail_Copy.entities.each do |edge|
@Current_Points << Geom::Point3d.new(edge.end.position)
end

#Put rail profile points on correct position and orientarion
@Side1_Point = @Serial_Points[@ID].offset(@Points_Axes[@ID][1], @Rails_Offset)
@Side1_Point = @Side1_Point.offset(@Points_Axes[@ID][0], @Up_Offset)

@Define_axes = Geom::Transformation.axes(@Side1_Point, @Points_Axes[@ID][1].normalize, @Points_Axes[@ID][4].normalize, @Points_Axes[@ID][0].normalize)
@Current_Points.each do |point|
point.transform!(@Define_axes)
end

#Make ends face
#I know its weird af to convert to string but sketchup was saying (1==1) as false. I swear :|
 if (((@ID == 0) or (@ID.to_s == (@Last_Point).to_s)) and (@MODE == "OPEN")) then @Rails1.entities.add_face @Current_Points; end

#Make lateral faces
 if ( (@ID != 0) or ((@MODE == "LOOP") and (@Last_ID.to_s == (@Last_Point-1).to_s)) )
  @Current_Points.each do |point|
   @SubID = @Current_Points.index(point)

   @Rails1.entities.add_face @Current_Points[@SubID-1], @Current_Points[@SubID], @Last_Points[@SubID-1]
   @Rails1.entities.add_face @Current_Points[@SubID], @Last_Points[@SubID-1], @Last_Points[@SubID]
  end
 end

#Make reference for next points
@Last_Points = @Current_Points
@Rail_Copy.erase!

@Last_ID = @ID
end

#Rails side 2 --------------------------------------
@Serial_Points.each do |position|
@ID = @Serial_Points.index(position)

#Add new rail profile
@Rail_Copy = @Rails2.entities.add_instance @Profile_def, Sketchup.active_model.axes.transformation

#Get rails profile points
@Current_Points = []
@Rail_Copy.entities.each do |edge|
@Current_Points << Geom::Point3d.new(edge.end.position)
end

#Put rail profile points on correct position and orientarion
@Side2_Point = @Serial_Points[@ID].offset(@Points_Axes[@ID][2], @Rails_Offset)
@Side2_Point = @Side2_Point.offset(@Points_Axes[@ID][0], @Up_Offset)

@Define_axes = Geom::Transformation.axes(@Side2_Point, @Points_Axes[@ID][1].normalize, @Points_Axes[@ID][4].normalize, @Points_Axes[@ID][0].normalize)
@Current_Points.each do |point|
point.transform!(@Define_axes)
end

#Make ends face
#I know its weird af to convert to string but sketchup was saying (1==1) as false. I swear :|
 if (((@ID == 0) or (@ID.to_s == (@Last_Point).to_s)) and (@MODE == "OPEN")) then @Rails2.entities.add_face @Current_Points; end

#Make lateral faces
 if ( (@ID != 0) or ((@MODE == "LOOP") and (@Last_ID.to_s == (@Last_Point-1).to_s)) )
  @Current_Points.each do |point|
   @SubID = @Current_Points.index(point)

   @Rails2.entities.add_face @Current_Points[@SubID-1], @Current_Points[@SubID], @Last_Points[@SubID-1]
   @Rails2.entities.add_face @Current_Points[@SubID], @Last_Points[@SubID-1], @Last_Points[@SubID]
  end
 end

#Make reference for next points
@Last_Points = @Current_Points
@Rail_Copy.erase!

@Last_ID = @ID
end

@Rail_Profile.erase!

#Hide rails edges #maybe should use (edge.soft = true)
@Rails1.entities.to_a.grep(Sketchup::Edge).each { |edge| edge.visible = false }
@Rails2.entities.to_a.grep(Sketchup::Edge).each { |edge| edge.visible = false }

@RailWay.material = @Rail_Color
#------------------------------------------------------------------------- <<<<<<<<<<<
Sketchup.active_model.commit_operation


print 'Make RailWay Along - Debug point 7/10 ok' #DEBUG POINT <<< ################################## <<<

#------------------------------------------------------------------------------------
@Timbers_Ammount = ((@Path_Length / @Timber_Spacing).to_i + 1)

@Point = 0
@Last_Position = @Serial_Points[@Point]
@Rest_of_Travel = @Timber_Spacing
@Rest_of_Edge = @Serial_Points[@Point].distance(@Serial_Points[@Point+1])

@Timber_Locations = [] # <<< [[position, ahead, up], [position, ahead, up]]
@Timber_Locations << [@Serial_Points[@Point], @Points_Axes[@Point][4], @Points_Axes[@Point][0], @Point]
@Timber_num = 0
#------------------------------------------------------------------------------------
until (@Timber_num == (@Timbers_Ammount - 1)) or (@Point > (@Serial_Points.length - 2))

   if (@Rest_of_Travel <= @Rest_of_Edge) #TRAVEL END ON THIS EDGE? <<<<<

      #Calculate timber position
      @Position = @Last_Position.offset(@Points_Axes[@Point][4], @Rest_of_Travel)
      @Last_Position = @Position #Update position historic

      @Rest_of_Edge -= @Rest_of_Travel #Update rest of edge
      @Rest_of_Travel = @Timber_Spacing #Reset travel goal since its ended

      #Calculate timber direction
      @Edge_Length = @Serial_Points[@Point].distance(@Serial_Points[@Point+1])
      @Factor = ((@Edge_Length - @Rest_of_Edge) / @Edge_Length)


      if (@Transition == "Linear") #TRANSITION MODE
      #Direct direction change (Option 1)
      @Direction = @Points_Axes[@Point][4]
      @Up_Vec = @Points_Axes[@Point][0]
      else
      #Gradient direction change (Option 2)
      @Direction = Geom::Vector3d.linear_combination(1-@Factor, @Points_Axes[@Point][4], @Factor, @Points_Axes[@Point+1][4])
      @Up_Vec = Geom::Vector3d.linear_combination(1-@Factor, @Points_Axes[@Point][0], @Factor, @Points_Axes[@Point+1][0])
      end
      
      
      #Register timber position values
      @Timber_Locations << [@Position, @Direction, @Up_Vec, @Point]
      @Timber_num += 1
   
   else #TRAVEL DON'T END ON CURRENT EDGE. <<<<<

      @Rest_of_Travel -= @Rest_of_Edge #Update rest of travel
      @Point += 1 #Pass to next edge
      @Last_Position = @Serial_Points[@Point]
      @Rest_of_Edge = @Serial_Points[@Point].distance(@Serial_Points[@Point+1]) #Reset rest of edge to next edge length

   end
end
#------------------------------------------------------------------------------------

print 'Make RailWay Along - Debug point 8/10 ok' #DEBUG POINT <<< ################################## <<<

Sketchup.active_model.start_operation "Railroad along op.3"
(#MAKE TIMBER -------------------------------------------------------------------------
model = Sketchup.active_model
entities = model.active_entities
@Timber = @RailWay.entities.add_group

#MAKE TIMBER BEVEL?
if (@Timber_Bevel == "Yes")
 #Define bevel value
 if (@Timber_Thickness < @Timber_Width) then @Bevel = @Timber_Thickness/@Bevel_Factor; else @Bevel = @Timber_Width/@Bevel_Factor; end
 @transf1 = Geom::Transformation.new([0, 0, -@Bevel])
 @transf2 = Geom::Transformation.new([0, 0, @Bevel])

 #UP SIDE
 @Pts = []
 @Pts[0] = [(@Timber_Lenght/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), 0]
 @Pts[1] = [-(@Timber_Lenght/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), 0]
 @Pts[2] = [-(@Timber_Lenght/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), 0]
 @Pts[3] = [(@Timber_Lenght/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), 0]
 @F1 = @Timber.entities.add_face @Pts

 #DOWNSIDE
 @Pts = []
 @Pts[0] = [(@Timber_Lenght/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[1] = [-(@Timber_Lenght/2.00 - @Bevel), (@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[2] = [-(@Timber_Lenght/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @Pts[3] = [(@Timber_Lenght/2.00 - @Bevel), -(@Timber_Width/2.00 - @Bevel), -@Timber_Thickness]
 @F2 = @Timber.entities.add_face @Pts
end

#BASIC TIMBER FORM
 @Pt1 = [(@Timber_Lenght/2.00), (@Timber_Width/2.00), 0]
 @Pt2 = [-(@Timber_Lenght/2.00), (@Timber_Width/2.00), 0]
 @Pt3 = [(@Timber_Lenght/2.00), -(@Timber_Width/2.00), 0]
 @Pt4 = [-(@Timber_Lenght/2.00), -(@Timber_Width/2.00), 0]

 @Line1 = @Timber.entities.add_line(@Pt1, @Pt2)
 @Line2 = @Timber.entities.add_line(@Pt3, @Pt4)
 @Line3 = @Timber.entities.add_line(@Pt1, @Pt3)
 @Line4 = @Timber.entities.add_line(@Pt2, @Pt4)
 @Base_Countourn1 = @Line1.all_connected

@Base_Face = @Timber.entities.add_face(@Base_Countourn1)
if (@Base_Face.normal != Z_AXIS) then @Base_Face.reverse!; end
@Base_Face.pushpull(-@Timber_Thickness, false)

#Finish bevel
if (@Timber_Bevel == "Yes")
 @Timber.entities.transform_entities(@transf1, @Base_Countourn1)
 
  #DOWNSIDE base Countourn2
 @Pt1 = [(@Timber_Lenght/2.00), (@Timber_Width/2.00), -@Timber_Thickness]
 @Pt2 = [-(@Timber_Lenght/2.00), (@Timber_Width/2.00), -@Timber_Thickness]
 @Pt3 = [(@Timber_Lenght/2.00), -(@Timber_Width/2.00), -@Timber_Thickness]
 @Pt4 = [-(@Timber_Lenght/2.00), -(@Timber_Width/2.00), -@Timber_Thickness]

 @Line1 = @Timber.entities.add_line(@Pt1, @Pt2)
 @Line2 = @Timber.entities.add_line(@Pt3, @Pt4)
 @Line3 = @Timber.entities.add_line(@Pt1, @Pt3)
 @Line4 = @Timber.entities.add_line(@Pt2, @Pt4)
 
 Sketchup.active_model.selection.clear
 Sketchup.active_model.selection.add(@Line1, @Line2, @Line3, @Line4)
 @Timber.entities.transform_entities(@transf2, Sketchup.active_model.selection)
 Sketchup.active_model.selection.clear
 Sketchup.active_model.selection.add(@Raw_Edges)
end

@Timber = @Timber.to_component
@Timber.name = "Timber"
@T_def = @Timber.definition
#------------------------------------------------------------------------------------
)

print 'Make RailWay Along - Debug point 9/10 ok' #DEBUG POINT <<< ################################## <<<

@All_Timbers = @RailWay.entities.add_group

@Timber_Locations.each do |timber|
   #entities.add_cpoint timber[0] #Timber position indicator
   @Timber_Position = timber[0].offset(timber[2], @Up_Offset)
   
   @Timber_Copy = @All_Timbers.entities.add_instance @T_def, (Geom::Transformation.axes(@Timber_Position, (timber[1].cross(timber[2])).normalize, timber[1], timber[2]))
   
   @Timber_Copy.name = "Timber"
   @Timber_Copy.definition = @T_def
   @Timber_Copy.material = @Timber_Color
end

@Timber.erase! #Delete original timber on origin point
#------------------------------------------------------------------------------------
Sketchup.active_model.commit_operation


print 'Make RailWay Along - Debug point 10/10 ok' #DEBUG POINT <<< ################################## <<<

#------------------------------------------------------------------------------------
@end_time = Time.now
@Total_Time = @end_time - @start_time

@Text1 = ("Success. (" + @Total_Time.round(3).to_s + "s)")
@Text2 = ("Path length: " + (@Path_Length / (1.00.m)).round(2).to_s + "m")
@Text3 = ("Timbers: " + @Timbers_Ammount.to_s + " | Edges: " + @Edges_Ammount.to_s)
@Text4 = ("Construction speed: " + ((@Path_Length / (1.00.m)) / @Total_Time).round(2).to_s + "m/s")

UI.messagebox(@Text1 + "\n" + @Text2 + "\n" + @Text3 + "\n" + @Text4)
print (@Text1 + "\n" + @Text2 + "\n" + @Text3 + "\n" + @Text4) + "\n\n"
#------------------------------------------------------------------------------------
end

end
